(function (testFns) {
    var breeze = testFns.breeze;
    var core = breeze.core;
    
    var Enum = core.Enum;
    var Event = core.Event;

    var MetadataStore = breeze.MetadataStore;
    var EntityManager = breeze.EntityManager;
    var EntityQuery = breeze.EntityQuery;
    var EntityKey = breeze.EntityKey;
    var EntityType = breeze.EntityType;
    var EntityState = breeze.EntityState;
    var EntityAction = breeze.EntityAction;
    var QueryOptions = breeze.QueryOptions;
    var SaveOptions = breeze.SaveOptions;
    var ValidationOptions = breeze.ValidationOptions;
    var MergeStrategy = breeze.MergeStrategy;
    var FetchStrategy = breeze.FetchStrategy;

    var newEm = testFns.newEm;

    module("entityManager", {
        setup: function () {
            testFns.setup();
        },
        teardown: function () {

        }
    });

    test("reject changes reverts an unmapped property - only unmapped property changed", 1, function () {
        var store = MetadataStore.importMetadata(newEm().metadataStore.exportMetadata());

        var originalTime = new Date(2013, 0, 1);
        var Customer = function () {
            this.lastTouched = originalTime;
        };

        store.registerEntityTypeCtor("Customer", Customer);

        var manager = newEm(store);

        // create a fake customer
        var cust = manager.createEntity("Customer", { CompanyName: "Acme" },
                   EntityState.Unchanged);
        var touched = cust.getProperty("lastTouched");

        // we change only the unmapped property (uncomment the next line and the test will pass)
        //cust.CompanyName("Beta");
        cust.setProperty("lastTouched", new Date(touched.getTime() + 60000));

        // cust.entityAspect.rejectChanges(); // roll back name change
        manager.rejectChanges(); // would have same effect. Obviously less granular
        var touchedAgain = cust.getProperty("lastTouched");
        ok(originalTime === touchedAgain,
            core.formatString("'lastTouched' unmapped property should be rolled back. Started as %1; now is %2",
            originalTime, cust.getProperty("lastTouched")));
    });


    test("export/import with nulls", function () {
        var queryOptions = new QueryOptions({
            mergeStrategy: MergeStrategy.OverwriteChanges,
            fetchStrategy: FetchStrategy.FromServer
        });
        var em = newEm();
        var pred = new breeze.Predicate("companyName", "!=" , null).and("city", "!=", null);
        var q = EntityQuery.from("Customers").where(pred).take(2)
            .using(MergeStrategy.OverwriteChanges);
        var val = Date.now().toString();
        stop();
        var exported;
        em.executeQuery(q).then(function (data) {
            var custs = data.results;
            custs[0].setProperty("companyName", null);
            custs[1].setProperty("city", null);
            exported = em.exportEntities(null, false);
            var em2 = newEm();
            em2.importEntities(exported);
            cust0x = em2.findEntityByKey(custs[0].entityAspect.getKey());
            ok(cust0x.getProperty("companyName") === null, "orig export companyName should be null");
            cust1x = em2.findEntityByKey(custs[1].entityAspect.getKey());
            ok(cust1x.getProperty("city") === null, "orig export city should be null");
            cust0x.setProperty("companyName", "Foo");
            cust1x.setProperty("city", "Foo");
            cust0x.entityAspect.acceptChanges();
            cust1x.entityAspect.acceptChanges();
            em2.importEntities(exported);
            ok(cust0x.getProperty("companyName") === null, "2nd import company should be null");
            ok(cust1x.getProperty("city") === null, "2nd import city should be null");
            
        }).fail(testFns.handleFail).fin(start);


    });

    test("relationship not resolved after import", function () {
        if (testFns.DEBUG_MONGO) {
            ok(true, "NA for Mongo - expand not YET supported");
            return;
        }

        var ds = new breeze.DataService({
            serviceName: "none",
            hasServerMetadata: false
        });

        var manager = new breeze.EntityManager({
            dataService: ds
        });


        var importData = { "dataService": { "serviceName": "/breeze/", "hasServerMetadata": true }, "saveOptions": { "allowConcurrentSaves": false }, "queryOptions": { "fetchStrategy": "FromServer", "mergeStrategy": "PreserveChanges" }, "validationOptions": { "validateOnAttach": true, "validateOnSave": true, "validateOnQuery": false, "validateOnPropertyChange": true }, "tempKeys": [], "entityGroupMap": { "shared_OrderMilestone:#Eagle.Models": { "entities": [{ "iD": 1, "name": "Order Placed", "setValue": 2, "sortIndex": 10, "optional": false, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 2, "name": "Design Complete", "setValue": 4, "sortIndex": 20, "optional": true, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 3, "name": "Design Approved", "setValue": 8, "sortIndex": 25, "optional": true, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 4, "name": "Order Approved", "setValue": 16, "sortIndex": 30, "optional": true, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 5, "name": "Released", "setValue": 32, "sortIndex": 40, "optional": false, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 6, "name": "Pre-Production Complete", "setValue": 64, "sortIndex": 110, "optional": true, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 7, "name": "Production Complete", "setValue": 128, "sortIndex": 120, "optional": true, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 8, "name": "Post-Production Complete", "setValue": 256, "sortIndex": 130, "optional": true, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 9, "name": "Built", "setValue": 512, "sortIndex": 150, "optional": false, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 10, "name": "Delivery Complete", "setValue": 1024, "sortIndex": 210, "optional": false, "entityAspect": { "entityState": "Unchanged" } }] }, "shared_OptionDef:#Eagle.Models": { "entities": [{ "name": "Amazon.AccessKeyId", "dataType": "Text", "categoryPath": "Amazon", "validOptionSet": "System", "defaultOptionValue": "AKIAID6WL2LRFUAT75SQ", "description": "Access Key Id for Amazon Web Services Simple Email Service", "isHidden": true, "isValidSystemOption": true, "isValidBusinessOption": false, "isValidStorefrontOption": false, "isValidEmployeeOption": false, "isValidCompanyOption": false, "isValidContactOption": false, "entityAspect": { "entityState": "Unchanged" } }, { "name": "Amazon.SecretAccessKey", "dataType": "Text", "categoryPath": "Amazon", "validOptionSet": "System", "defaultOptionValue": "SRSjJYe5+pwad8xVtU/bM8dnZ1j+uHSHUN3bF+7O", "description": "Secret Access Key for Amazon Web Services Simple Email Service", "isHidden": true, "isValidSystemOption": true, "isValidBusinessOption": false, "isValidStorefrontOption": false, "isValidEmployeeOption": false, "isValidCompanyOption": false, "isValidContactOption": false, "entityAspect": { "entityState": "Unchanged" } }, { "name": "BackgroundColor", "dataType": "Text", "categoryPath": "UI\\Color", "validOptionSet": "None", "defaultOptionValue": "Gray", "description": "Sets the background color of the pages.", "isHidden": false, "isValidSystemOption": false, "isValidBusinessOption": false, "isValidStorefrontOption": false, "isValidEmployeeOption": false, "isValidCompanyOption": false, "isValidContactOption": false, "entityAspect": { "entityState": "Unchanged" } }, { "name": "CyriousOlarkEnabled", "dataType": "Boolean", "categoryPath": "Olark", "validOptionSet": "System", "defaultOptionValue": "True", "description": "Is Olark Enabled for Employees", "isHidden": true, "isValidSystemOption": true, "isValidBusinessOption": false, "isValidStorefrontOption": false, "isValidEmployeeOption": false, "isValidCompanyOption": false, "isValidContactOption": false, "entityAspect": { "entityState": "Unchanged" } }, { "name": "CyriousOlarkSiteID", "dataType": "Text", "categoryPath": "Olark", "validOptionSet": "System", "defaultOptionValue": "4195-861-10-2091", "description": "Cyrious Olark Site iD", "isHidden": true, "isValidSystemOption": true, "isValidBusinessOption": false, "isValidStorefrontOption": false, "isValidEmployeeOption": false, "isValidCompanyOption": false, "isValidContactOption": false, "entityAspect": { "entityState": "Unchanged" } }, { "name": "FontColor", "dataType": "Text", "categoryPath": "UI\\Color", "validOptionSet": "None", "defaultOptionValue": "Black", "description": "Sets the default font color.", "isHidden": false, "isValidSystemOption": false, "isValidBusinessOption": false, "isValidStorefrontOption": false, "isValidEmployeeOption": false, "isValidCompanyOption": false, "isValidContactOption": false, "entityAspect": { "entityState": "Unchanged" } }, { "name": "IsFreeAccount", "dataType": "Boolean", "categoryPath": "Cyrious\\Billing", "validOptionSet": "None", "defaultOptionValue": "0", "description": "Indicates this is a complimentary account", "isHidden": true, "isValidSystemOption": false, "isValidBusinessOption": false, "isValidStorefrontOption": false, "isValidEmployeeOption": false, "isValidCompanyOption": false, "isValidContactOption": false, "entityAspect": { "entityState": "Unchanged" } }, { "name": "OlarkEnabled", "dataType": "Boolean", "categoryPath": "Olark", "validOptionSet": "System, Business", "defaultOptionValue": "False", "description": "Is Olark Enabled for Contacts", "isHidden": true, "isValidSystemOption": true, "isValidBusinessOption": true, "isValidStorefrontOption": false, "isValidEmployeeOption": false, "isValidCompanyOption": false, "isValidContactOption": false, "entityAspect": { "entityState": "Unchanged" } }, { "name": "OlarkSiteID", "dataType": "Text", "categoryPath": "Olark", "validOptionSet": "System, Business", "defaultOptionValue": "", "description": "Business Olark Site iD", "isHidden": true, "isValidSystemOption": true, "isValidBusinessOption": true, "isValidStorefrontOption": false, "isValidEmployeeOption": false, "isValidCompanyOption": false, "isValidContactOption": false, "entityAspect": { "entityState": "Unchanged" } }, { "name": "Version", "dataType": "Numeric", "categoryPath": "Software", "validOptionSet": "None", "optionListValues": "", "defaultOptionValue": "1.00", "description": "Indicates which version of Eagle to run.", "isHidden": true, "isValidSystemOption": false, "isValidBusinessOption": false, "isValidStorefrontOption": false, "isValidEmployeeOption": false, "isValidCompanyOption": false, "isValidContactOption": false, "entityAspect": { "entityState": "Unchanged" } }] }, "shared_TaxClassification:#Eagle.Models": { "entities": [{ "iD": 1, "name": "Goods", "tIC": "00000", "description": "General Goods.  Any product or service which does not fit within the other defined categories.", "defaultIsTaxable": true, "sortIndex": 10, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 20, "name": "Services", "tIC": "10010", "description": "Charges by the seller for any services necessary to complete the sale (other than delivery or installation) means any separately contracted for, separately stated on the invoice or similar document, and the amount is not the sales price.", "defaultIsTaxable": false, "sortIndex": 30, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 30, "name": "Installation", "tIC": "10040", "description": "Charges by the seller for any services necessary to complete the sale (other than delivery) means any separately contracted for, separately stated on the invoice or similar document, and the amount is not the sales price.", "defaultIsTaxable": false, "sortIndex": 20, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 40, "name": "Shipping", "tIC": "11010", "description": "Transportation, shipping, postage, and similar charges.", "defaultIsTaxable": false, "sortIndex": 40, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 42, "name": "Shipping - Pickup Available", "tIC": "90022", "description": "Shipping Exemption: Some states, such as Florida, provide an exemption on shipping charges. This TIC returns no tax on shipping, but the transaction must meet the following criteria.\r\n\r\n  The shipping charge must be separately stated on the invoice.\r\n  The purchaser must have the option to pick up the item, thus avoiding the shipping charge.\r\n\r\nIf the shipping charges are mandatory then the shipping charges are taxable.", "defaultIsTaxable": false, "sortIndex": 42, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 44, "name": "Handling", "tIC": "11000", "description": "Handling, crating, packing, preparation for mailing or delivery, and similar charges.", "defaultIsTaxable": false, "sortIndex": 44, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 80, "name": "Materials - General", "tIC": "94001", "description": "\"General Materials\" does not include\r\n\r\n  Lumber;\r\n  Engineered Materials;", "defaultIsTaxable": true, "sortIndex": 80, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 82, "name": "Materials - Lumber", "tIC": "94002", "description": "Wood that is cut, shaped, and treated for construction or fabrication. After purchase, lumber is usually processed further to take the form desired by the user and typically becomes part of a final product. Intended uses include, but are not limited to, buildings, structures, and furniture.\r\n\r\nLumber includes:\r\n  Stud;\r\n  Plank;\r\n  Beam;\r\n  Pole;\r\n  Post;", "defaultIsTaxable": true, "sortIndex": 82, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 84, "name": "Materials - Engineered", "tIC": "94003", "description": "Construction materials made of composites or blended materials shaped or molded similar to conventional construction materials.\r\n\r\nEngineered Materials includes\r\n  Gypsum-bonded composite materials (fiber reinforced gypsum);\r\n  Cement-bonded composite materials;\r\n  Ceramic bonded composite material;\r\n  Wood thermoplastic composite materials including plastic lumber and decking;\r\n  Hardie Fiber Cement Siding;", "defaultIsTaxable": true, "sortIndex": 84, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 120, "name": "Software - Online", "tIC": "30070", "description": "Remote Access to means \"Prewritten computer software\" accessed by the purchaser remotely (not downloaded).\r\nRemote Access to prewriten software does not include subscription-based access to newspapers (TIC: 93113), periodicals (TIC: 93117), online gaming (TIC: 93104), Audio Streaming (TIC: 31090), Video Streaming (TIC: 31060), or any other electronically delivered (downloaded) goods.", "defaultIsTaxable": true, "sortIndex": 120, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 122, "name": "Software - Support", "tIC": "91010", "description": "Troubleshooting and error resolution support via phone, billed at a flat rate regardless of the amount of support actually used", "defaultIsTaxable": false, "sortIndex": 122, "entityAspect": { "entityState": "Unchanged" } }] }, "shared_OrderStatus:#Eagle.Models": { "entities": [{ "iD": 0, "name": "Draft", "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 1, "name": "PreApproval", "sortIndex": 10, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 2, "name": "WIP", "sortIndex": 20, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 3, "name": "Built", "sortIndex": 30, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 4, "name": "A/R", "sortIndex": 40, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 5, "name": "Closed", "sortIndex": 50, "entityAspect": { "entityState": "Unchanged" } }, { "iD": 6, "name": "Voided", "sortIndex": 99, "entityAspect": { "entityState": "Unchanged" } }] }, "shared_Origin:#Eagle.Models": { "entities": [{ "iD": 2, "name": "Association - Business Group", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 4, "name": "Association - Chamber", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 6, "name": "Association - Other", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 8, "name": "Association - Personal", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 10, "name": "Cold Call - Phone", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 12, "name": "Cold Call - Visit", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 14, "name": "Internet - Ad", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 16, "name": "Internet - Blog", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 18, "name": "Internet - Other", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 20, "name": "Internet - Search Engine", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 22, "name": "Internet - Social Media", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 24, "name": "Internet - Web Site", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 26, "name": "Marketing - Direct Email", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 28, "name": "Marketing - Direct Mail", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 30, "name": "Marketing - Other", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 32, "name": "Marketing - Outdoor", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 34, "name": "Marketing - Telemarketing", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 36, "name": "Marketing - Vehicle Wrap", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 38, "name": "Newspaper", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 40, "name": "Other 1", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 42, "name": "Other 2", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 44, "name": "Other 3", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 46, "name": "Other 4", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 48, "name": "Other 5", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 50, "name": "Radio", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 52, "name": "Referral - Business", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 54, "name": "Referral - Other", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 56, "name": "Referral - Personal", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 58, "name": "Referral - Related Company", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 60, "name": "Television", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 62, "name": "Walk In", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 64, "name": "Yellow Pages", "entityAspect": { "entityState": "Unchanged" } }] }, "shared_Industry:#Eagle.Models": { "entities": [{ "iD": 10, "name": "Accounting", "sIC": "6920", "nAIC": "541211", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 11, "name": "Advertising", "sIC": "7310", "nAIC": "541810", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 12, "name": "Agriculture, Forestry, Fishing & Hunting", "sIC": "7999", "nAIC": "110000", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 13, "name": "Air Conditioning/Htg/Refrigeration", "sIC": "3585", "nAIC": "238220", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 14, "name": "Amusement Places", "sIC": "9321", "nAIC": "713110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 15, "name": "Apartments", "sIC": "6820", "nAIC": "531311", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 16, "name": "Arts Organizations & Info", "sIC": "9499", "nAIC": "813319", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 17, "name": "Associations", "sIC": "9499", "nAIC": "813410", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 18, "name": "Athletic Organizations", "sIC": "7990", "nAIC": "713990", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 19, "name": "Automobile Parts & Supplies", "sIC": "4530", "nAIC": "441310", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 20, "name": "Automotive", "sIC": "7549", "nAIC": "551100", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 21, "name": "Automotive - Repair", "sIC": "4520", "nAIC": "811111", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 22, "name": "Automotive - Sales", "sIC": "7549", "nAIC": "551100", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 23, "name": "Banking", "sIC": "6419", "nAIC": "522110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 24, "name": "Beauty Salons", "sIC": "9602", "nAIC": "812112", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 25, "name": "Beer & Ale", "sIC": "4722", "nAIC": "445310", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 26, "name": "Building Restoration & Preservation", "sIC": "1751", "nAIC": "236118", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 27, "name": "Casino-Hotel", "sIC": "7011", "nAIC": "722120", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 28, "name": "Charitable Organization", "sIC": "9499", "nAIC": "813219", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 29, "name": "Child Care Service", "sIC": "8890", "nAIC": "624410", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 30, "name": "Churches", "sIC": "9491", "nAIC": "813110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 31, "name": "Cigars, Cigarettes & Tobacco", "sIC": "5993", "nAIC": "312230", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 32, "name": "Cleaning Services", "sIC": "8121", "nAIC": "561720", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 33, "name": "Coffee & Tea Products", "sIC": "2095", "nAIC": "333241", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 34, "name": "Computer Software", "sIC": "5820", "nAIC": "511210", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 35, "name": "Computers-Service & Repair", "sIC": "3312", "nAIC": "811212", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 36, "name": "Condominiums", "sIC": "6820", "nAIC": "531311", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 37, "name": "Construction", "sIC": "1522", "nAIC": "236220", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 38, "name": "Convenience Stores", "sIC": "4711", "nAIC": "445120", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 39, "name": "Electronic Equipment & Supplies", "sIC": "3313", "nAIC": "443142", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 40, "name": "Entertainment", "sIC": "7929", "nAIC": "710000", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 41, "name": "Environmental & Ecological", "sIC": "9499", "nAIC": "813312", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 42, "name": "Exercise & Physical Fitness", "sIC": "4763", "nAIC": "451110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 43, "name": "Florists-Retail", "sIC": "4773", "nAIC": "453110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 44, "name": "Food Products & Manufacturing", "sIC": "2099", "nAIC": "311991", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 45, "name": "Games & Game Supplies", "sIC": "3944", "nAIC": "339930", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 46, "name": "Garden Centers", "sIC": "4773", "nAIC": "444220", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 47, "name": "General Merchandise-Retail", "sIC": "4530", "nAIC": "452990", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 48, "name": "Government", "sIC": "9199", "nAIC": "920000", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 49, "name": "Grocers-Retail", "sIC": "4711", "nAIC": "445110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 50, "name": "Health & Diet Foods-Retail", "sIC": "4721", "nAIC": "446191", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 51, "name": "Health Clubs, Studios & Gyms", "sIC": "9311", "nAIC": "713940", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 52, "name": "Health Spas", "sIC": "9609", "nAIC": "812199", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 53, "name": "Hospitals", "sIC": "8610", "nAIC": "622110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 54, "name": "Hotels & Motels", "sIC": "5510", "nAIC": "721110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 55, "name": "Individual", "sIC": "8890", "nAIC": "624190", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 56, "name": "Industrial", "sIC": "3599", "nAIC": "333999", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 57, "name": "Insurance", "sIC": "6622", "nAIC": "524210", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 58, "name": "Internet Service", "sIC": "6201", "nAIC": "541511", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 59, "name": "Lawn Services", "sIC": "8130", "nAIC": "561730", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 60, "name": "Lawyers", "sIC": "6910", "nAIC": "541110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 61, "name": "Manufacturing", "sIC": "3999", "nAIC": "310000", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 62, "name": "Mobile Telephone Equipment & Services", "sIC": "6120", "nAIC": "517210", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 63, "name": "Museums and Art Galleries", "sIC": "9102", "nAIC": "712110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 64, "name": "Music & Live Entertainment", "sIC": "9000", "nAIC": "711130", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 65, "name": "Non-Profit Organizations", "sIC": "8641", "nAIC": "81331 ", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 66, "name": "Nonclassified Establishment", "sIC": "9999", "nAIC": "999999", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 67, "name": "Nursing & Convalescent Homes", "sIC": "8710", "nAIC": "623110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 68, "name": "Office Buildings & Parks", "sIC": "9131", "nAIC": "236220", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 69, "name": "Oils & Petroleum Manufacturing", "sIC": "2911", "nAIC": "324199", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 71, "name": "Pharmacies", "sIC": "5912", "nAIC": "466110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 72, "name": "Physicians", "sIC": "8620", "nAIC": "621111", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 73, "name": "Plastics & Plastic Products", "sIC": "2899", "nAIC": "325211", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 74, "name": "Political Organizations", "sIC": "9492", "nAIC": "813940", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 75, "name": "Printing", "sIC": "2759", "nAIC": "323111", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 76, "name": "Professional Organization", "sIC": "9411", "nAIC": "813910", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 78, "name": "Radio Stations & Broadcasting", "sIC": "6010", "nAIC": "515111", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 79, "name": "Real Estate - Residential & Commercial", "sIC": "6910", "nAIC": "541110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 80, "name": "Recreational Vehicles", "sIC": "4510", "nAIC": "441210", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 81, "name": "Religious Organizations", "sIC": "9491", "nAIC": "813110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 82, "name": "Religious Schools", "sIC": "9491", "nAIC": "813110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 83, "name": "Restaurants", "sIC": "5610", "nAIC": "722511", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 84, "name": "Retail - Food", "sIC": "4711", "nAIC": "445110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 85, "name": "Retail - Gas", "sIC": "4711", "nAIC": "447110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 86, "name": "Retail - General", "sIC": "4719", "nAIC": "452111", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 87, "name": "Retirement Communities", "sIC": "8730", "nAIC": "623312", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 88, "name": "Schools", "sIC": "8299", "nAIC": "610000", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 89, "name": "Schools - Elementary & Secondary School", "sIC": "8510", "nAIC": "611110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 90, "name": "Schools - University", "sIC": "8530", "nAIC": "611310", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 91, "name": "Schools-Nursery & Kindergarten", "sIC": "8510", "nAIC": "611110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 92, "name": "Service Stations-Gasoline", "sIC": "5541", "nAIC": "238290", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 93, "name": "Shopping Centers & Malls", "sIC": "6810", "nAIC": "531120", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 94, "name": "Sign Company", "sIC": "3993", "nAIC": "339950", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 95, "name": "Sporting Goods-Retail", "sIC": "4763", "nAIC": "451110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 96, "name": "Stadiums, Arenas & Athletic Organizations", "sIC": "7941", "nAIC": "711319", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 97, "name": "Television Stations & Broadcasting", "sIC": "6020", "nAIC": "515120", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 98, "name": "Theatres-Movie", "sIC": "5914", "nAIC": "512131", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 99, "name": "Tourist Information", "sIC": "7990", "nAIC": "561591", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 100, "name": "Transportation", "sIC": "4789", "nAIC": "480000", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 101, "name": "Trucking", "sIC": "4923", "nAIC": "484121", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 102, "name": "Unknown", "sIC": "9999", "nAIC": "0     ", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 103, "name": "Utilities", "sIC": "9631", "nAIC": "220000", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 104, "name": "Veterinarians", "sIC": "7500", "nAIC": "541940", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 105, "name": "Youth Organizations & Centers", "sIC": "8890", "nAIC": "624110", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 106, "name": "Zoos and Gardens", "sIC": "8422", "nAIC": "712310", "entityAspect": { "entityState": "Unchanged" } }] }, "shared_AssetType:#Eagle.Models": { "entities": [{ "iD": 1, "assetType": "Text", "displayText": "Text Files", "sortIndex": 1, "defaultMediaType": "text/plain", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 2, "assetType": "Document", "displayText": "Documents", "sortIndex": 2, "defaultMediaType": "application/x-cyr.unidentified.document", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 3, "assetType": "Image", "displayText": "Image Files", "sortIndex": 3, "defaultMediaType": "image/x-cyr.unidentified.image", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 4, "assetType": "Artwork", "displayText": "Artwork Files", "sortIndex": 4, "defaultMediaType": "application/x-cyr.unidentified.artwork", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 5, "assetType": "Video", "displayText": "Video Files", "sortIndex": 5, "defaultMediaType": "video/x-cyr.unidentified.video", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 6, "assetType": "Audio", "displayText": "Audio Files", "sortIndex": 6, "defaultMediaType": "audio/x-cyr.unidentified.audio", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 7, "assetType": "Web", "displayText": "Web Files", "sortIndex": 7, "defaultMediaType": "text/x-cyr.unidentified.web", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 8, "assetType": "Binary", "displayText": "Binary Data", "sortIndex": 8, "defaultMediaType": "application/x-cyr.unidentified.binary", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 9, "assetType": "XML", "displayText": "XML Data", "sortIndex": 9, "defaultMediaType": "text/xml", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 10, "assetType": "Other", "displayText": "Other Data", "sortIndex": 10, "defaultMediaType": "application/x-cyr.unidentified.other", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 11, "assetType": "Unknown", "displayText": "Unknown File Type", "sortIndex": 11, "defaultMediaType": "application/x-cyr.unidentified.unknown", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 12, "assetType": "Font", "displayText": "Font", "sortIndex": 12, "defaultMediaType": "application/font-woff", "entityAspect": { "entityState": "Unchanged" } }] }, "shared_AssetMediaType:#Eagle.Models": { "entities": [{ "assetTypeID": 1, "iD": 1, "mediaType": "text/plain", "displayText": "Text File", "sortIndex": 104, "isPartialMatchOK": false, "defaultExtensions": "txt;log", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 1, "iD": 11, "mediaType": "text/csv", "displayText": "Comma Delimited File", "sortIndex": 102, "isPartialMatchOK": false, "defaultExtensions": "csv", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 2, "iD": 2, "mediaType": "application/pdf", "displayText": "PDF", "sortIndex": 208, "isPartialMatchOK": false, "defaultExtensions": "pdf", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 2, "iD": 3, "mediaType": "application/vnd.ms-excel", "displayText": "Excel (2003)", "sortIndex": 204, "isPartialMatchOK": false, "defaultExtensions": "xls", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 2, "iD": 4, "mediaType": "application/vnd.ms-powerpoint", "displayText": "Powerpoint (2003)", "sortIndex": 212, "isPartialMatchOK": false, "defaultExtensions": "ppt", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 2, "iD": 5, "mediaType": "application/vnd.ms-word", "displayText": "Word (2003)", "sortIndex": 220, "isPartialMatchOK": false, "defaultExtensions": "doc", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 2, "iD": 6, "mediaType": "application/vnd.opemxmlformats-officedocument.spreadsheetml.sheet", "displayText": "Excel", "sortIndex": 202, "isPartialMatchOK": false, "defaultExtensions": "xlsx", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 2, "iD": 7, "mediaType": "application/vnd.opemxmlformats-officedocument.wordprocessingml.document", "displayText": "Word", "sortIndex": 218, "isPartialMatchOK": false, "defaultExtensions": "docx", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 2, "iD": 8, "mediaType": "application/vnd.opemxmlformats-officedocument.spreadsheetml.presentation", "displayText": "Powerpoint", "sortIndex": 210, "isPartialMatchOK": false, "defaultExtensions": "pptx", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 2, "iD": 9, "mediaType": "application/vnd.", "displayText": "Vendor Document", "sortIndex": 250, "isPartialMatchOK": true, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 2, "iD": 10, "mediaType": "application/x-latext", "displayText": "LaTex File", "sortIndex": 206, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 2, "iD": 12, "mediaType": "text/vcard", "displayText": "vCard", "sortIndex": 216, "isPartialMatchOK": false, "defaultExtensions": "vcard", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 2, "iD": 53, "mediaType": "application/x-cyr.unidentified.document", "displayText": "Unknown Document", "sortIndex": 214, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 3, "iD": 17, "mediaType": "image/gif", "displayText": "GIF", "sortIndex": 308, "isPartialMatchOK": false, "defaultExtensions": "gif", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 3, "iD": 18, "mediaType": "image/jpeg", "displayText": "JPeG", "sortIndex": 310, "isPartialMatchOK": false, "defaultExtensions": "jpg;jpeg", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 3, "iD": 20, "mediaType": "image/png", "displayText": "PNG", "sortIndex": 312, "isPartialMatchOK": false, "defaultExtensions": "png", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 3, "iD": 21, "mediaType": "image/svg+xml", "displayText": "SVG", "sortIndex": 316, "isPartialMatchOK": false, "defaultExtensions": "svg", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 3, "iD": 22, "mediaType": "image/tiff", "displayText": "TIFF", "sortIndex": 318, "isPartialMatchOK": false, "defaultExtensions": "tif;tiff", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 3, "iD": 23, "mediaType": "image/", "displayText": "Image (Other)", "sortIndex": 352, "isPartialMatchOK": true, "defaultExtensions": "bmp;cdr;ai;", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 3, "iD": 24, "mediaType": "application/postscript", "displayText": "EPS", "sortIndex": 306, "isPartialMatchOK": false, "defaultExtensions": "eps;ps", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 3, "iD": 54, "mediaType": "image/x-cyr.unidentified.image", "displayText": "Unknown Image", "sortIndex": 320, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 4, "iD": 13, "mediaType": "application/x-cyr.artwork", "displayText": "Artwork File", "sortIndex": 362, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 4, "iD": 14, "mediaType": "application/x-cyr.artwork.template", "displayText": "Artwork Template File", "sortIndex": 364, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 4, "iD": 15, "mediaType": "application/x-cyr.stockartwork", "displayText": "Stock Artwork File", "sortIndex": 366, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 4, "iD": 62, "mediaType": "application/x-cyr.unidentified.artwork", "displayText": "Unknown Artwork", "sortIndex": 369, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 5, "iD": 25, "mediaType": "video/", "displayText": "Video File", "sortIndex": 450, "isPartialMatchOK": true, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 5, "iD": 55, "mediaType": "video/x-cyr.unidentified.video", "displayText": "Unknown Video", "sortIndex": 402, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 6, "iD": 26, "mediaType": "audio/", "displayText": "Audio File", "sortIndex": 550, "isPartialMatchOK": true, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 6, "iD": 56, "mediaType": "audio/x-cyr.unidentified.audio", "displayText": "Unknown Audio", "sortIndex": 502, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 7, "iD": 27, "mediaType": "application/ecmascript", "displayText": "Javascript", "sortIndex": 614, "isPartialMatchOK": false, "defaultExtensions": "js", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 7, "iD": 29, "mediaType": "application/json", "displayText": "JSON Data", "sortIndex": 620, "isPartialMatchOK": false, "defaultExtensions": "jsn;json", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 7, "iD": 30, "mediaType": "application/javascript", "displayText": "Javascript (deprecated)", "sortIndex": 616, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 7, "iD": 32, "mediaType": "text/css", "displayText": "CSS File", "sortIndex": 602, "isPartialMatchOK": false, "defaultExtensions": "css", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 7, "iD": 33, "mediaType": "text/html", "displayText": "HTML Web Page", "sortIndex": 612, "isPartialMatchOK": false, "defaultExtensions": "htm;html", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 7, "iD": 34, "mediaType": "text/javascript", "displayText": "Javascript (deprecated)", "sortIndex": 618, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 7, "iD": 36, "mediaType": "application/x-www-form-urlencoded", "displayText": "Encoded Web Form", "sortIndex": 604, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 7, "iD": 57, "mediaType": "text/x-cyr-unidentified-web", "displayText": "Unknown Web", "sortIndex": 622, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 8, "iD": 37, "mediaType": "application/zip", "displayText": "Compressed ZIP file", "sortIndex": 712, "isPartialMatchOK": false, "defaultExtensions": "zip", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 8, "iD": 38, "mediaType": "application/gzip", "displayText": "Compressed GZIP file", "sortIndex": 704, "isPartialMatchOK": false, "defaultExtensions": "gz;gzip", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 8, "iD": 39, "mediaType": "application/x-stuffit", "displayText": "Compressed StuffIt File", "sortIndex": 708, "isPartialMatchOK": false, "defaultExtensions": "stuff", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 8, "iD": 40, "mediaType": "application/x-rar-compressed", "displayText": "Compressed RAR file", "sortIndex": 706, "isPartialMatchOK": false, "defaultExtensions": "rar", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 8, "iD": 41, "mediaType": "application/tar", "displayText": "Compressed TAR file", "sortIndex": 710, "isPartialMatchOK": false, "defaultExtensions": "tar", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 8, "iD": 42, "mediaType": "application/octet-stream", "displayText": "Binary File", "sortIndex": 702, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 8, "iD": 58, "mediaType": "application/x-cyr.unidentified.binary", "displayText": "Unknown Binary", "sortIndex": 714, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 9, "iD": 43, "mediaType": "application/xml", "displayText": "XML File", "sortIndex": 806, "isPartialMatchOK": false, "defaultExtensions": "xml", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 9, "iD": 44, "mediaType": "application/xml", "displayText": "XML Related File", "sortIndex": 850, "isPartialMatchOK": true, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 9, "iD": 45, "mediaType": "application/soap+xml", "displayText": "SOAP XML File", "sortIndex": 802, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 9, "iD": 46, "mediaType": "text/XML", "displayText": "XML File (deprecated)", "sortIndex": 808, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 9, "iD": 59, "mediaType": "text/xml", "displayText": "Unknown XML", "sortIndex": 804, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 10, "iD": 47, "mediaType": "message/", "displayText": "Message File", "sortIndex": 954, "isPartialMatchOK": true, "defaultExtensions": "eml", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 10, "iD": 48, "mediaType": "model/", "displayText": "3D Model File", "sortIndex": 950, "isPartialMatchOK": true, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 10, "iD": 49, "mediaType": "multi-part/", "displayText": "Multi-Part File", "sortIndex": 956, "isPartialMatchOK": true, "defaultExtensions": "mime", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 10, "iD": 50, "mediaType": "application/x-", "displayText": "Custom File Type", "sortIndex": 952, "isPartialMatchOK": true, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 10, "iD": 60, "mediaType": "application/x-cyr.unidentified.other", "displayText": "Unknown Other", "sortIndex": 902, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 11, "iD": 51, "mediaType": "", "displayText": "Unknown File Type", "sortIndex": 1050, "isPartialMatchOK": true, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 11, "iD": 61, "mediaType": "application/x-cyr.unidentified.unknown", "displayText": "Unknown Unknown", "sortIndex": 1002, "isPartialMatchOK": false, "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 12, "iD": 28, "mediaType": "application/font-woff", "displayText": "Font (WOFF)", "sortIndex": 610, "isPartialMatchOK": false, "defaultExtensions": "woff", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 12, "iD": 31, "mediaType": "application/ogg", "displayText": "Font (OGG)", "sortIndex": 606, "isPartialMatchOK": false, "defaultExtensions": "ogg", "entityAspect": { "entityState": "Unchanged" } }, { "assetTypeID": 12, "iD": 35, "mediaType": "application/x-font-ttf", "displayText": "Font (TrueType)", "sortIndex": 608, "isPartialMatchOK": false, "defaultExtensions": "ttf;tt", "entityAspect": { "entityState": "Unchanged" } }] }, "shared_Unit:#Eagle.Models": { "entities": [{ "iD": 0, "dimType": "N", "dimensionality": 1, "shortName": "      ", "name": "(none)", "unitsPerBaseUnit": 1, "baseUnitsPerUnit": 1, "textMatch": "none;", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 1, "dimType": "D", "dimensionality": 1, "shortName": "ea    ", "name": "each", "unitsPerBaseUnit": 1, "baseUnitsPerUnit": 1, "textMatch": "Each;ea;", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 2, "dimType": "L", "dimensionality": 1, "shortName": "in    ", "name": "Inch", "unitsPerBaseUnit": 1, "baseUnitsPerUnit": 1, "textMatch": "Inch;Inches;In;", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 3, "dimType": "L", "dimensionality": 1, "shortName": "ft    ", "name": "Foot", "unitsPerBaseUnit": 0.0833333333, "baseUnitsPerUnit": 12, "textMatch": "Foot;Feet;Ft;", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 4, "dimType": "L", "dimensionality": 1, "shortName": "yd    ", "name": "Yard", "unitsPerBaseUnit": 0.0277777778, "baseUnitsPerUnit": 36, "textMatch": "Yard;Yards;Yd;", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 5, "dimType": "L", "dimensionality": 1, "shortName": "m     ", "name": "Meter", "unitsPerBaseUnit": 0.0254, "baseUnitsPerUnit": 39.37007874, "textMatch": "Meter;Metre;Meters;Metres;m", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 6, "dimType": "L", "dimensionality": 1, "shortName": "cm    ", "name": "Centimeter", "unitsPerBaseUnit": 2.54, "baseUnitsPerUnit": 0.393700787, "textMatch": "Centimeter;Centimetre;Centimeters;Centimeters;cm", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 7, "dimType": "L", "dimensionality": 1, "shortName": "mm    ", "name": "Millimeter", "unitsPerBaseUnit": 25.4, "baseUnitsPerUnit": 0.039370079, "textMatch": "Millimeter;Millimeters;Millimetre;Millimetres;mm", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 20, "dimType": "V", "dimensionality": 1, "shortName": "ml    ", "name": "Milliliters", "unitsPerBaseUnit": 16.39, "baseUnitsPerUnit": 0.061012813, "textMatch": "Milliliter;Millilitre;Milliliters;Millilitres;mm", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 21, "dimType": "V", "dimensionality": 1, "shortName": "l     ", "name": "Liter", "unitsPerBaseUnit": 0.01639, "baseUnitsPerUnit": 61.012812691, "textMatch": "Liter;Litre;Liters;Litres;L", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 22, "dimType": "V", "dimensionality": 1, "shortName": "Gal   ", "name": "Gallon", "unitsPerBaseUnit": 0.004329, "baseUnitsPerUnit": 231.000231, "textMatch": "Gallon;Gallons;gal;gals", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 23, "dimType": "V", "dimensionality": 1, "shortName": "Qt    ", "name": "Quart", "unitsPerBaseUnit": 0.01732, "baseUnitsPerUnit": 57.736720554, "textMatch": "Quart;Quarts;Qt;Qts", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 24, "dimType": "V", "dimensionality": 1, "shortName": "Pt    ", "name": "Pint", "unitsPerBaseUnit": 0.03463, "baseUnitsPerUnit": 28.876696506, "textMatch": "Pint;Pints;Pt;Pts", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 28, "dimType": "W", "dimensionality": 1, "shortName": "g     ", "name": "Gram", "unitsPerBaseUnit": 28.35, "baseUnitsPerUnit": 0.035273369, "textMatch": "Gram;Grams;g", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 29, "dimType": "W", "dimensionality": 1, "shortName": "kg    ", "name": "Kilogram", "unitsPerBaseUnit": 0.02835, "baseUnitsPerUnit": 35.273368607, "textMatch": "Kilogram;Kilograms;kg;kgs", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 30, "dimType": "W", "dimensionality": 1, "shortName": "Oz    ", "name": "Ounce", "unitsPerBaseUnit": 1, "baseUnitsPerUnit": 1, "textMatch": "Ounce;Ounces;Oz;", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 31, "dimType": "W", "dimensionality": 1, "shortName": "Lb    ", "name": "Pound", "unitsPerBaseUnit": 0.0625, "baseUnitsPerUnit": 16, "textMatch": "Pound;Pounds;Lb;Lbs", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 33, "dimType": "T", "dimensionality": 1, "shortName": "sec   ", "name": "Second", "unitsPerBaseUnit": 1, "baseUnitsPerUnit": 1, "textMatch": "Second;Seconds;sec", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 34, "dimType": "T", "dimensionality": 1, "shortName": "min   ", "name": "Minute", "unitsPerBaseUnit": 0.0166666667, "baseUnitsPerUnit": 60, "textMatch": "Minute;Minutes;min;", "entityAspect": { "entityState": "Unchanged" } }, { "iD": 35, "dimType": "T", "dimensionality": 1, "shortName": "Hour  ", "name": "Hour", "unitsPerBaseUnit": 0.0002777778, "baseUnitsPerUnit": 3600, "textMatch": "Hour;Hours;Hr;", "entityAspect": { "entityState": "Unchanged" } }] }, "CompanyLocator:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 101, "companyID": 1, "name": "Work", "categoryType": "Phone", "categorySubType": "Other", "formattedLocator": "2255555555", "isPrimary": false, "isPhone": true, "isEmail": false, "isURL": false, "isTwitter": false, "isFacebook": false, "isText": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 162, "companyID": 1, "name": "Twitter", "categoryType": "Twitter", "formattedLocator": "@twitter", "isPrimary": false, "isPhone": false, "isEmail": false, "isURL": false, "isTwitter": true, "isFacebook": false, "isText": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 163, "companyID": 1, "name": "Email", "categoryType": "Email", "formattedLocator": "email@place.co", "isPrimary": false, "isPhone": false, "isEmail": true, "isURL": false, "isTwitter": false, "isFacebook": false, "isText": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 164, "companyID": 1, "name": "URL", "categoryType": "URL", "formattedLocator": "url.co", "isPrimary": false, "isPhone": false, "isEmail": false, "isURL": true, "isTwitter": false, "isFacebook": false, "isText": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }] }, "ContactLocator:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 331, "contactID": 331, "name": "Twitter", "categoryType": "Twitter", "formattedLocator": "@twitter", "isPrimary": false, "isPhone": false, "isEmail": false, "isURL": false, "isTwitter": true, "isFacebook": false, "isText": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 332, "contactID": 331, "name": "Phone", "categoryType": "Phone", "categorySubType": "Work", "formattedLocator": "1234565784", "isPrimary": false, "isPhone": true, "isEmail": false, "isURL": false, "isTwitter": false, "isFacebook": false, "isText": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }] }, "Contact:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 331, "eTag": 1, "isActive": true, "companyID": 1, "name": "Scott St. Cyr", "firstName": "Scott", "lastName": "St. Cyr", "gender": true, "hasAdditionalImages": false, "requireOrderApproval": false, "canApproveOrders": false, "isPrimary": true, "isBilling": false, "isShipping": false, "imageURL": "content/images/Contacts/contact3.png", "primaryImageID": 0, "entityAspect": { "entityState": "Unchanged" } }] }, "Company:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 1, "eTag": 6, "isActive": true, "storefrontID": 2, "name": "Cyrious Software", "hasAdditionalImages": false, "isBasedOnContact": false, "companyNumber": 1, "flagsCount": 0, "hasFlags": false, "requireOrderApprovalByDefault": false, "isOnCreditHold": false, "hasCreditAccount": false, "creditUsed": 0, "creditAvailable": 0, "defaultDiscountRate": 10, "isOrderPaidByPayor": false, "hasDiscount": true, "imageURL": "/asset/company/1/images/7a66ae07-521a-4483-9daf-dce2e3511d14.jpg", "primaryImageID": 0, "entityAspect": { "entityState": "Unchanged" } }] }, "Employee:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 1, "eTag": 3, "isActive": true, "name": "Cyrious Admin", "firstName": "Cyrious", "lastName": "Admin", "hasAdditionalImages": false, "position": "Admin", "canApproveInternal": false, "canApproveCustomers": false, "isPrimary": false, "isBilling": false, "canApproveAllStorefronts": false, "imageURL": "/asset/employee/1/images/38b0b5c2-8315-4455-b1ef-ab0a48b3e9a0.jpg", "primaryImageID": 0, "entityAspect": { "entityState": "Unchanged" } }] }, "OrderItemAttribute:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 387915, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 16735, "name": "Image_Width", "itemDataType": "Numeric", "isReadOnly": true, "isRequired": false, "value": "12000", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387927, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 7149, "name": "Quantity", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "value": "5", "isList": false, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387941, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 7149, "name": "Quantity", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "value": "1", "isList": false, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387926, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 7150, "name": "Size", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "value": "3:5", "isList": true, "listType": "SizeList", "listSelectionValue": "0", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387942, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 7150, "name": "Size", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "value": "3:5", "isList": true, "listType": "SizeList", "listSelectionValue": "0", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387925, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 7151, "name": "Area_Units", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387943, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 7151, "name": "Area_Units", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387924, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 7152, "name": "Units", "itemDataType": "Numeric", "isReadOnly": true, "isRequired": false, "value": "Inch", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387944, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 7152, "name": "Units", "itemDataType": "Numeric", "isReadOnly": true, "isRequired": false, "value": "Inch", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387923, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 7153, "name": "Total_Area", "itemDataType": "Numeric", "isReadOnly": true, "isRequired": false, "value": "1200", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387945, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 7153, "name": "Total_Area", "itemDataType": "Numeric", "isReadOnly": true, "isRequired": false, "value": "240", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387922, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 7154, "name": "Area", "itemDataType": "Numeric", "isReadOnly": true, "isRequired": false, "value": "240", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387946, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 7154, "name": "Area", "itemDataType": "Numeric", "isReadOnly": true, "isRequired": false, "value": "240", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387921, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 7155, "name": "Width", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "value": "20", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387947, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 7155, "name": "Width", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "value": "20", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387920, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 7156, "name": "Height", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "value": "12", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387948, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 7156, "name": "Height", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "value": "12", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387919, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 7157, "name": "Sides", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "value": "1", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387949, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 7157, "name": "Sides", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "value": "1", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387916, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 16734, "name": "Image_Height", "itemDataType": "Numeric", "isReadOnly": true, "isRequired": false, "value": "7200", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387951, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 16734, "name": "Image_Height", "itemDataType": "Numeric", "isReadOnly": true, "isRequired": false, "value": "7200", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387917, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 7159, "name": "Material", "itemDataType": "Text", "isReadOnly": false, "isRequired": false, "value": "Reinforced", "isList": false, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387952, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 7159, "name": "Material", "itemDataType": "Text", "isReadOnly": false, "isRequired": false, "value": "", "isList": false, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387918, "orderID": 0, "orderItemID": 38831, "itemID": 617, "itemAttributeID": 7158, "name": "Price", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "isList": false, "listType": "Area", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387953, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 7158, "name": "Price", "itemDataType": "Numeric", "isReadOnly": false, "isRequired": false, "isList": false, "listType": "Area", "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 387950, "orderID": 0, "orderItemID": 38833, "itemID": 617, "itemAttributeID": 16735, "name": "Image_Width", "itemDataType": "Numeric", "isReadOnly": true, "isRequired": false, "value": "12000", "isList": false, "listType": "NotDefined", "entityAspect": { "entityState": "Unchanged" } }] }, "ItemAttribute:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 7149, "itemID": 617, "isSystem": true, "name": "Quantity", "inputType": "Label", "dataType": "Numeric", "isReadOnly": false, "isRequired": false, "defaultValueFx": "1", "isList": false, "listData": "[]", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 7150, "itemID": 617, "isSystem": true, "name": "Size", "inputType": "Label", "dataType": "Numeric", "isReadOnly": false, "isRequired": false, "defaultValueFx": "0", "isList": true, "listType": "SizeList", "listData": "[{\"label\":\"3:5\",\"h\":12,\"w\":20,\"sort\":2,\"old\":true},{\"label\":\"2:3\",\"h\":10,\"w\":15,\"sort\":1,\"old\":true},{\"label\":\"1:1\",\"h\":10,\"w\":10,\"sort\":0,\"old\":true},{\"label\":\"10:19\",\"h\":15,\"w\":33,\"sort\":3,\"old\":true}]", "validationFx": "{\"allowNoSize\":false,\"useLength\":false}", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 7151, "itemID": 617, "isSystem": true, "name": "Area_Units", "inputType": "Label", "dataType": "Numeric", "isReadOnly": false, "isRequired": false, "defaultValueFx": "", "isList": false, "listType": "NotDefined", "listData": "", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 7152, "itemID": 617, "isSystem": true, "name": "Units", "inputType": "Label", "dataType": "Numeric", "isReadOnly": false, "isRequired": false, "defaultValueFx": "", "isList": false, "listType": "NotDefined", "listData": "", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 7153, "itemID": 617, "isSystem": true, "name": "Total_Area", "inputType": "Label", "dataType": "Numeric", "isReadOnly": false, "isRequired": false, "defaultValueFx": "", "isList": false, "listType": "NotDefined", "listData": "", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 7154, "itemID": 617, "isSystem": true, "name": "Area", "inputType": "Label", "dataType": "Numeric", "isReadOnly": false, "isRequired": false, "defaultValueFx": "", "isList": false, "listType": "NotDefined", "listData": "", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 7155, "itemID": 617, "isSystem": true, "name": "Width", "inputType": "Label", "dataType": "Numeric", "isReadOnly": false, "isRequired": false, "defaultValueFx": "", "isList": false, "listType": "NotDefined", "listData": "", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 7156, "itemID": 617, "isSystem": true, "name": "Height", "inputType": "Label", "dataType": "Numeric", "isReadOnly": false, "isRequired": false, "defaultValueFx": "", "isList": false, "listType": "NotDefined", "listData": "", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 7157, "itemID": 617, "isSystem": true, "name": "Sides", "inputType": "Label", "dataType": "Numeric", "isReadOnly": false, "isRequired": false, "defaultValueFx": "false", "isList": false, "listType": "NotDefined", "listData": "", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 16734, "itemID": 617, "isSystem": true, "name": "Image_Height", "inputType": "Label", "dataType": "Numeric", "isReadOnly": false, "isRequired": false, "defaultValueFx": "", "isList": false, "listType": "NotDefined", "listData": "", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 7159, "itemID": 617, "isSystem": false, "name": "Material", "inputType": "Label", "dataType": "Text", "isReadOnly": false, "isRequired": false, "isList": false, "listData": "{\"notInList\":{\"value\":\"Not in list\",\"oneTime\":12,\"each\":0,\"percentage\":0,\"oneSided\":0,\"twoSided\":0},\"list\":[{\"value\":\"All - Weather\",\"oneTime\":2,\"each\":0,\"percentage\":0,\"oneSided\":3,\"twoSided\":3},{\"value\":\"Reinforced\",\"oneTime\":0,\"each\":0,\"percentage\":0,\"oneSided\":6,\"twoSided\":6}]}", "validationFx": "{\"allowNoAnswer\":true}", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 7158, "itemID": 617, "isSystem": true, "name": "Price", "inputType": "Label", "dataType": "Numeric", "isReadOnly": false, "isRequired": false, "defaultValueFx": "{\"inSearch\":false,\"type\":0,\"text\":\"Call for Quote\",\"price\":0,\"starting\":0}", "isList": false, "listType": "Area", "listData": "{\"list\":[{\"oneSided\":3,\"twoSided\":4,\"perItem\":0,\"setup\":0}]}", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 16735, "itemID": 617, "isSystem": true, "name": "Image_Width", "inputType": "Label", "dataType": "Numeric", "isReadOnly": false, "isRequired": false, "defaultValueFx": "", "isList": false, "listType": "NotDefined", "listData": "", "isShownInGrid": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }] }, "OrderItem:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 38833, "orderID": 35775, "eTag": 0, "orderNumber": 0, "itemNumber": 2, "orderItemNumber": "0-002", "itemID": 617, "itemName": "Custom Flag", "description": "", "quantity": 1, "priceSubTotal": 5, "priceDiscount": 0.5, "priceTotal": 4.5, "discountRate": 10, "discountOV": false, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 38831, "orderID": 35775, "eTag": 0, "orderNumber": 0, "itemNumber": 1, "orderItemNumber": "0-001", "itemID": 617, "itemName": "Custom Flag", "description": "", "quantity": 5, "priceSubTotal": 75, "priceDiscount": 7.5, "priceTotal": 67.5, "discountRate": 10, "discountOV": false, "entityAspect": { "entityState": "Unchanged" } }] }, "ItemDef:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 617, "eTag": 14, "name": "Custom Flag", "categoryID": 2, "categoryPath": "Signage > Digital Printing", "searchDescription": "", "useSearchDescription": false, "description": "", "wODescription": "", "imageURL": "/asset/itemdef/617/images/7e8090ad-f1bc-449c-915f-cf24c55582aa.png", "hasAdditionalImages": false, "tags": "flag", "sKUs": "", "productType": "Manufactured", "isStockItem": false, "isShippingItem": false, "hasImages": false, "isNegativeInvOK": false, "isInvDisplayed": false, "taxClassificationID": 1, "isExcludedFromDiscounts": false, "entryFormID": 1, "hasUnits": true, "inputUnitID": 2, "pricingUnitID": 3, "isActive": true, "primaryImageID": 0, "entityAspect": { "entityState": "Unchanged" } }] }, "OrderShipmentAddress:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 33125, "shipmentID": 33115, "name": "Home", "isPrimary": true, "isBilling": true, "isShipping": true, "isResidential": false, "isGeo": false, "isVerified": false, "address": "1111 America Way", "city": "Schenectady", "state": "New York", "postalCode": "12345", "country": "US ", "hasCoordinates": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }] }, "OrderShipmentLocator:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 46120, "shipmentID": 33115, "name": "Email", "categoryType": "Email", "formattedLocator": "breadslinger@bread.com", "isPrimary": false, "isPhone": false, "isEmail": true, "isURL": false, "isTwitter": false, "isFacebook": false, "isText": false, "sortIndex": 1, "entityAspect": { "entityState": "Unchanged" } }, { "bID": 1, "iD": 46121, "shipmentID": 33115, "name": "Phone", "categoryType": "Phone", "formattedLocator": "(225) 555-1234", "isPrimary": true, "isPhone": true, "isEmail": false, "isURL": false, "isTwitter": false, "isFacebook": false, "isText": false, "sortIndex": 0, "entityAspect": { "entityState": "Unchanged" } }] }, "OrderShipment:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 33115, "orderID": 35775, "eTag": 0, "itemID": 0, "itemName": "Something", "priceSubTotal": 0, "priceDiscount": 0, "priceTotal": 0, "discountRate": 0, "discountOV": false, "isShipped": false, "isArrived": false, "contactName": "Brandon Readlinger", "entityAspect": { "entityState": "Unchanged" } }] }, "OrderHeader:#Eagle.Models": { "entities": [{ "bID": 1, "iD": 35775, "eTag": 0, "isDraft": true, "orderNumber": 35775, "status": "Draft", "orderingCompanyID": 1, "orderingContactID": 331, "billingCompanyID": 1, "billingContactID": 331, "priceSubTotal": 80, "priceDiscount": 0, "priceShipping": 0, "priceTaxes": 7.2, "priceTotal": 87.2, "paymentTotal": 0, "creditAppliedTotal": 0, "writeOffTotal": 0, "balanceDue": 87.2, "dtCreated": "2013-07-19T13:44:00.000Z", "enteredByEmployeeID": 1, "isFirmDueDate": false, "taxAmountOV": false, "taxRateOV": false, "taxRate": 9, "deliveryType": "LocalPickup", "discountRate": 0, "entityAspect": { "entityState": "Unchanged" } }] } }, "isEdit": true, "metadataStore": "{\"metadataVersion\":\"1.0.5\",\"namingConvention\":\"camelCase\",\"localQueryComparisonOptions\":\"caseInsensitiveSQL\",\"dataServices\":[{\"serviceName\":\"/breeze/\",\"hasServerMetadata\":true,\"jsonResultsAdapter\":\"webApi_default\",\"useJsonp\":false}],\"structuralTypes\":[{\"shortName\":\"AlertQueue\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"AlertQueues\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"dtCreated\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"deliveryAttempts\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"dtLastAttempt\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtNextAttempt\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"contactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"notificationMethod\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.NotificationMethodType\"},{\"name\":\"fromAddress\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"toAddress\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}]},{\"name\":\"separateMessages\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"cCAddress\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"bCCAddress\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"subject\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"body\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}]},{\"name\":\"uRLRegistrationID\",\"dataType\":\"Guid\",\"validators\":[{\"name\":\"guid\"}]},{\"name\":\"deliveryNotesStr\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"deliveryNotes\",\"dataType\":\"Undefined\",\"validators\":[{\"name\":\"none\"}],\"rawTypeName\":\"Edm.List`1\"}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AlertQueue_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"contact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AlertQueue_Contact\",\"foreignKeyNames\":[\"bID\",\"contactID\"]},{\"name\":\"employee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AlertQueue_Employee\",\"foreignKeyNames\":[\"bID\",\"employeeID\"]},{\"name\":\"uRLRegistration\",\"entityTypeName\":\"URLRegistration:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AlertQueue_URLRegistration\",\"foreignKeyNames\":[\"bID\",\"uRLRegistrationID\"]},{\"name\":\"attachments\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"AssetLink_Alert\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"Business\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"Businesses\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"hasAdditionalImages\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"useDefaultTheme\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"themeID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"defaultSubDomain\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":18,\"validators\":[{\"name\":\"required\"},{\"maxLength\":18,\"name\":\"maxLength\"}]},{\"name\":\"customURL\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"taxRate\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"primaryImageID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"imageURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"theme\",\"entityTypeName\":\"Theme:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Business_Theme\",\"foreignKeyNames\":[\"themeID\"]},{\"name\":\"primaryImage\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Business_PrimaryImage\",\"foreignKeyNames\":[\"bID\",\"primaryImageID\"]},{\"name\":\"businessAddresses\",\"entityTypeName\":\"BusinessAddress:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"BusinessAddress_Business\",\"invForeignKeyNames\":[\"bID\"]},{\"name\":\"businessLocators\",\"entityTypeName\":\"BusinessLocator:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"BusinessLocator_Business\",\"invForeignKeyNames\":[\"bID\"]},{\"name\":\"companyTaxExemptions\",\"entityTypeName\":\"CompanyTaxExemption:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"CompanyTaxExemption_Business\",\"invForeignKeyNames\":[\"bID\"]},{\"name\":\"employees\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Employee_Business\",\"invForeignKeyNames\":[\"bID\"]},{\"name\":\"itemDefs\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemDef_Business\",\"invForeignKeyNames\":[\"bID\"]},{\"name\":\"orderHeaders\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderHeader_Business\",\"invForeignKeyNames\":[\"bID\"]},{\"name\":\"orderMilestoneLinks\",\"entityTypeName\":\"OrderMilestoneLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderMilestoneLink_Business\",\"invForeignKeyNames\":[\"bID\"]},{\"name\":\"orderItemMilestoneLinks\",\"entityTypeName\":\"OrderItemMilestoneLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderItemMilestoneLink_Business\",\"invForeignKeyNames\":[\"bID\"]},{\"name\":\"storefronts\",\"entityTypeName\":\"Storefront:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Storefront_Business\",\"invForeignKeyNames\":[\"bID\"]},{\"name\":\"storefrontGroups\",\"entityTypeName\":\"StorefrontGroup:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"StorefrontGroup_Business\",\"invForeignKeyNames\":[\"bID\"]},{\"name\":\"optionSets\",\"entityTypeName\":\"OptionSet:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OptionSet_Business\",\"invForeignKeyNames\":[\"bID\"]}]},{\"shortName\":\"Theme\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"Themes\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"bID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"cSSID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"headerHTMLID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"messageHTMLID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"promotionHTMLID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"isShared\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]}],\"navigationProperties\":[{\"name\":\"businesses\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Business_Theme\",\"invForeignKeyNames\":[\"iD\"]},{\"name\":\"storefronts\",\"entityTypeName\":\"Storefront:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Storefront_Theme\",\"invForeignKeyNames\":[\"iD\"]},{\"name\":\"storefrontGroups\",\"entityTypeName\":\"StorefrontGroup:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"StorefrontGroup_DefaultTheme\",\"invForeignKeyNames\":[\"iD\"]},{\"name\":\"cSSAsset\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Theme_CSSAsset\",\"foreignKeyNames\":[\"bID\",\"cSSID\"]},{\"name\":\"headerAsset\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Theme_HeaderAsset\",\"foreignKeyNames\":[\"bID\",\"headerHTMLID\"]},{\"name\":\"messageAsset\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Theme_MessageAsset\",\"foreignKeyNames\":[\"bID\",\"messageHTMLID\"]},{\"name\":\"promotionAsset\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Theme_PromotionAsset\",\"foreignKeyNames\":[\"bID\",\"promotionHTMLID\"]}]},{\"shortName\":\"Storefront\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"Storefronts\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"storefrontGroupID\",\"dataType\":\"Byte\",\"validators\":[{\"name\":\"byte\"}]},{\"name\":\"isEmployeeStorefront\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isPublicStorefront\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"hasAdditionalImages\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"themeID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"forceLoginToAccess\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"forceLoginToViewPricing\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"requireOrderApprovalByDefault\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"uRLMasks\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":4096,\"validators\":[{\"name\":\"required\"},{\"maxLength\":4096,\"name\":\"maxLength\"}]},{\"name\":\"employeeRepID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"companyContactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"imageURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"primaryImageID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Storefront_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"companies\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Company_Storefront\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"theme\",\"entityTypeName\":\"Theme:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Storefront_Theme\",\"foreignKeyNames\":[\"themeID\"]},{\"name\":\"storefrontGroup\",\"entityTypeName\":\"StorefrontGroup:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Storefront_StorefrontGroup\",\"foreignKeyNames\":[\"bID\",\"storefrontGroupID\"]},{\"name\":\"employeeRep\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Storefront_EmployeeRep\",\"foreignKeyNames\":[\"bID\",\"employeeRepID\"]},{\"name\":\"companyContact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Storefront_CompanyContact\",\"foreignKeyNames\":[\"bID\",\"companyContactID\"]},{\"name\":\"storefrontItemLinks\",\"entityTypeName\":\"StorefrontItemLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"StorefrontItemLink_Storefront\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"storefrontEmployeeApproverLinks\",\"entityTypeName\":\"StorefrontEmployeeApproverLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"StorefrontEmployeeApproverLink_Storefront\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"storefrontContactApproverLinks\",\"entityTypeName\":\"StorefrontContactApproverLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"StorefrontContactApproverLink_Storefront\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"optionSets\",\"entityTypeName\":\"OptionSet:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OptionSet_Storefront\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"primaryImage\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Storefront_PrimaryImage\",\"foreignKeyNames\":[\"bID\",\"primaryImageID\"]}]},{\"shortName\":\"Company\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"Companies\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"storefrontID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"hasAdditionalImages\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isBasedOnContact\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"companyNumber\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"salespersonID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"industryID\",\"dataType\":\"Byte\",\"validators\":[{\"name\":\"byte\"}]},{\"name\":\"originID\",\"dataType\":\"Byte\",\"validators\":[{\"name\":\"byte\"}]},{\"name\":\"flagsCount\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"hasFlags\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"requireOrderApprovalByDefault\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isOnCreditHold\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"hasCreditAccount\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"creditLimit\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"creditUsed\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"creditAvailable\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"creditNotes\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"defaultDiscountRate\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"payorCompanyID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"payorContactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"isOrderPaidByPayor\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"hasDiscount\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"imageURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"primaryImageID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"industry\",\"entityTypeName\":\"shared_Industry:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Company_Industry\",\"foreignKeyNames\":[\"industryID\"]},{\"name\":\"origin\",\"entityTypeName\":\"shared_Origin:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Company_Origin\",\"foreignKeyNames\":[\"originID\"]},{\"name\":\"storefront\",\"entityTypeName\":\"Storefront:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Company_Storefront\",\"foreignKeyNames\":[\"bID\",\"storefrontID\"]},{\"name\":\"payorCompany\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Company_PayorCompany\",\"foreignKeyNames\":[\"bID\",\"payorCompanyID\"]},{\"name\":\"payorContact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Company_PayorContact\",\"foreignKeyNames\":[\"bID\",\"payorContactID\"]},{\"name\":\"salesperson\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Company_Salesperson\",\"foreignKeyNames\":[\"bID\",\"salespersonID\"]},{\"name\":\"companyActivities\",\"entityTypeName\":\"CompanyActivity:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"CompanyActivity_Company\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"companyAddresses\",\"entityTypeName\":\"CompanyAddress:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"CompanyAddress_Company\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"companyLocators\",\"entityTypeName\":\"CompanyLocator:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"CompanyLocator_Company\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"companyTaxExemptions\",\"entityTypeName\":\"CompanyTaxExemption:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"CompanyTaxExemption_Company\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"contacts\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Contact_Company\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"orderHeadersBilledToMe\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderHeader_BillingCompany\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"orderHeadersOrderedByMe\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderHeader_OrderingCompany\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"payingForCompanies\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Company_PayorCompany\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"optionSets\",\"entityTypeName\":\"OptionSet:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OptionSet_Company\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"primaryImage\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Company_PrimaryImage\",\"foreignKeyNames\":[\"bID\",\"primaryImageID\"]}]},{\"shortName\":\"shared_Industry\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_Industries\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"sIC\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":12,\"validators\":[{\"name\":\"required\"},{\"maxLength\":12,\"name\":\"maxLength\"}]},{\"name\":\"nAIC\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":12,\"validators\":[{\"name\":\"required\"},{\"maxLength\":12,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"companies\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Company_Industry\",\"invForeignKeyNames\":[\"iD\"]}]},{\"shortName\":\"shared_Origin\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_Origins\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"companies\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Company_Origin\",\"invForeignKeyNames\":[\"iD\"]}]},{\"shortName\":\"Contact\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"Contacts\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"companyID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"firstName\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"lastName\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"gender\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"hasAdditionalImages\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"position\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"birthdayMonth\",\"dataType\":\"Byte\",\"validators\":[{\"name\":\"byte\"}]},{\"name\":\"birthdayDay\",\"dataType\":\"Byte\",\"validators\":[{\"name\":\"byte\"}]},{\"name\":\"requireOrderApproval\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"canApproveOrders\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"defaultImageID\",\"dataType\":\"Byte\",\"validators\":[{\"name\":\"byte\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isBilling\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isShipping\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"imageURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"primaryImageID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"company\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Contact_Company\",\"foreignKeyNames\":[\"bID\",\"companyID\"]},{\"name\":\"companyActivities\",\"entityTypeName\":\"CompanyActivity:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"CompanyActivity_Contact\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"contactAddresses\",\"entityTypeName\":\"ContactAddress:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ContactAddress_Contact\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"contactLocators\",\"entityTypeName\":\"ContactLocator:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ContactLocator_Contact\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"orderHeadersBilledToMe\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderHeader_BillingContact\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"orderHeadersOrderedByMe\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderHeader_OrderingContact\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"orderHeadersEnteredByMe\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderHeader_EnteredByContact\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"users\",\"entityTypeName\":\"User:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"User_Contact\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"payingForCompanies\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Company_PayorContact\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"storefrontContactApproverLinks\",\"entityTypeName\":\"StorefrontContactApproverLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"StorefrontContactApproverLink_Contact\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"storefrontGroupContactApproverLinks\",\"entityTypeName\":\"StorefrontGroupContactApproverLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"StorefrontGroupContactApproverLink_Contact\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"optionSets\",\"entityTypeName\":\"OptionSet:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OptionSet_Contact\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"primaryImage\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Contact_PrimaryImage\",\"foreignKeyNames\":[\"bID\",\"primaryImageID\"]}]},{\"shortName\":\"CompanyActivity\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"CompanyActivities\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"companyID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"activityType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.ActivityType\"},{\"name\":\"description\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"notes\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"dtStart\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"dtEnd\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"dtCompleted\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"contactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"isCompleted\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isAllDay\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]}],\"navigationProperties\":[{\"name\":\"company\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"CompanyActivity_Company\",\"foreignKeyNames\":[\"bID\",\"companyID\"]},{\"name\":\"contact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"CompanyActivity_Contact\",\"foreignKeyNames\":[\"bID\",\"contactID\"]}]},{\"shortName\":\"ContactAddress\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"ContactAddresses\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"contactID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"formattedAddress\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isBilling\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isShipping\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isResidential\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isGeo\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isVerified\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"address\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"city\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"state\",\"dataType\":\"String\",\"maxLength\":32,\"validators\":[{\"maxLength\":32,\"name\":\"maxLength\"}]},{\"name\":\"postalCode\",\"dataType\":\"String\",\"maxLength\":16,\"validators\":[{\"maxLength\":16,\"name\":\"maxLength\"}]},{\"name\":\"country\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":3,\"validators\":[{\"name\":\"required\"},{\"maxLength\":3,\"name\":\"maxLength\"}]},{\"name\":\"hasCoordinates\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"coordinates\",\"dataType\":\"Undefined\",\"validators\":[{\"name\":\"none\"}],\"rawTypeName\":\"Edm.Geography\"},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"contact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ContactAddress_Contact\",\"foreignKeyNames\":[\"bID\",\"contactID\"]}]},{\"shortName\":\"ContactLocator\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"ContactLocators\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"contactID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"categoryType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.LocatorCategoryType\"},{\"name\":\"categorySubType\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PhoneType\"},{\"name\":\"formattedLocator\",\"dataType\":\"String\",\"maxLength\":512,\"validators\":[{\"maxLength\":512,\"name\":\"maxLength\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isPhone\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isEmail\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isURL\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isTwitter\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isFacebook\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isText\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"contact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ContactLocator_Contact\",\"foreignKeyNames\":[\"bID\",\"contactID\"]}]},{\"shortName\":\"OrderHeader\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"OrderHeaders\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isDraft\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"orderNumber\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"description\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"status\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.OrderStatus\"},{\"name\":\"stationID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"orderingCompanyID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"orderingContactID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"billingCompanyID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"billingContactID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"priceSubTotal\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"priceDiscount\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"priceShipping\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"priceTaxes\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"priceTotal\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"paymentTotal\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"creditAppliedTotal\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"writeOffTotal\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"balanceDue\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"dtCreated\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"dtApproved\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtReleased\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtBuilt\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtSaleCompleted\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtClosed\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtVoided\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"salespersonID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"enteredByEmployeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"enteredByContactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"dtCustomerDue\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtProductionDue\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"isFirmDueDate\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"pONumber\",\"dataType\":\"String\",\"maxLength\":128,\"validators\":[{\"maxLength\":128,\"name\":\"maxLength\"}]},{\"name\":\"taxAmountOV\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"taxRateOV\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"taxRate\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"taxData\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"deliveryType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.DeliveryType\"},{\"name\":\"deliverToContact\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"deliverToCompany\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"discountRate\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"internalNotes\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"customerNotes\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"verificationHash\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderHeader_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"billingCompany\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderHeader_BillingCompany\",\"foreignKeyNames\":[\"bID\",\"billingCompanyID\"]},{\"name\":\"orderingCompany\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderHeader_OrderingCompany\",\"foreignKeyNames\":[\"bID\",\"orderingCompanyID\"]},{\"name\":\"billingContact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderHeader_BillingContact\",\"foreignKeyNames\":[\"bID\",\"billingContactID\"]},{\"name\":\"orderingContact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderHeader_OrderingContact\",\"foreignKeyNames\":[\"bID\",\"orderingContactID\"]},{\"name\":\"enteredByContact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderHeader_EnteredByContact\",\"foreignKeyNames\":[\"bID\",\"enteredByContactID\"]},{\"name\":\"enteredByEmployee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderHeader_EnteredByEmployee\",\"foreignKeyNames\":[\"bID\",\"enteredByEmployeeID\"]},{\"name\":\"orderItems\",\"entityTypeName\":\"OrderItem:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderItem_OrderHeader\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"orderShipments\",\"entityTypeName\":\"OrderShipment:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderShipment_OrderHeader\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"orderItemMilestoneLinks\",\"entityTypeName\":\"OrderItemMilestoneLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderItemMilestoneLink_Order\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"orderMilestoneLinks\",\"entityTypeName\":\"OrderMilestoneLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderMilestoneLink_Order\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"itemInventoryLogs\",\"entityTypeName\":\"ItemInventoryLog:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemInventoryLog_OrderHeader\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"Employee\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"Employees\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"firstName\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"lastName\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"hasAdditionalImages\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"position\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"birthdayMonth\",\"dataType\":\"Byte\",\"validators\":[{\"name\":\"byte\"}]},{\"name\":\"birthdayDay\",\"dataType\":\"Byte\",\"validators\":[{\"name\":\"byte\"}]},{\"name\":\"canApproveInternal\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"canApproveCustomers\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isBilling\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"canApproveAllStorefronts\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"imageURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"primaryImageID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Employee_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"employeeAddresses\",\"entityTypeName\":\"EmployeeAddress:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"EmployeeAddress_Employee\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"employeeLocators\",\"entityTypeName\":\"EmployeeLocator:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"EmployeeLocator_Employee\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"users\",\"entityTypeName\":\"User:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"User_Employee\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"orderHeadersEnteredByMe\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderHeader_EnteredByEmployee\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"storefrontEmployeeApproverLinks\",\"entityTypeName\":\"StorefrontEmployeeApproverLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"StorefrontEmployeeApproverLink_Employee\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"storefrontGroupEmployeeApproverLinks\",\"entityTypeName\":\"StorefrontGroupEmployeeApproverLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"StorefrontGroupEmployeeApproverLink_Employee\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"optionSets\",\"entityTypeName\":\"OptionSet:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OptionSet_Employee\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"itemInventoryLogs\",\"entityTypeName\":\"ItemInventoryLog:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemInventoryLog_Employee\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"companies\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Company_Salesperson\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"primaryImage\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Employee_PrimaryImage\",\"foreignKeyNames\":[\"bID\",\"primaryImageID\"]}]},{\"shortName\":\"EmployeeAddress\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"EmployeeAddresses\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"formattedAddress\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isBilling\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isShipping\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isResidential\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isGeo\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isVerified\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"address\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"city\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"state\",\"dataType\":\"String\",\"maxLength\":32,\"validators\":[{\"maxLength\":32,\"name\":\"maxLength\"}]},{\"name\":\"postalCode\",\"dataType\":\"String\",\"maxLength\":16,\"validators\":[{\"maxLength\":16,\"name\":\"maxLength\"}]},{\"name\":\"country\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":3,\"validators\":[{\"name\":\"required\"},{\"maxLength\":3,\"name\":\"maxLength\"}]},{\"name\":\"hasCoordinates\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"coordinates\",\"dataType\":\"Undefined\",\"validators\":[{\"name\":\"none\"}],\"rawTypeName\":\"Edm.Geography\"},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"employee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"EmployeeAddress_Employee\",\"foreignKeyNames\":[\"bID\",\"employeeID\"]}]},{\"shortName\":\"EmployeeLocator\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"EmployeeLocators\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"categoryType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.LocatorCategoryType\"},{\"name\":\"categorySubType\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PhoneType\"},{\"name\":\"formattedLocator\",\"dataType\":\"String\",\"maxLength\":512,\"validators\":[{\"maxLength\":512,\"name\":\"maxLength\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isPhone\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isEmail\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isURL\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isTwitter\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isFacebook\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isText\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"employee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"EmployeeLocator_Employee\",\"foreignKeyNames\":[\"bID\",\"employeeID\"]}]},{\"shortName\":\"User\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"Users\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"userID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"userName\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"roleName\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"contactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"isBusinessEmployee\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isCyriousEmployee\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isAccountContact\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isAdmin\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"mustResetPassword\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"dtLastLogin\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"cartID\",\"dataType\":\"Int64\",\"validators\":[{\"name\":\"integer\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]}],\"navigationProperties\":[{\"name\":\"contact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"User_Contact\",\"foreignKeyNames\":[\"bID\",\"contactID\"]},{\"name\":\"employee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"User_Employee\",\"foreignKeyNames\":[\"bID\",\"employeeID\"]},{\"name\":\"cart\",\"entityTypeName\":\"Cart:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"User_Cart\",\"foreignKeyNames\":[\"bID\",\"cartID\"]}]},{\"shortName\":\"UserAuth\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"UserAuths\",\"dataProperties\":[{\"name\":\"authID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"userName\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]}]},{\"shortName\":\"Cart\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"Carts\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int64\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"integer\"}]},{\"name\":\"token\",\"dataType\":\"Guid\",\"validators\":[{\"name\":\"guid\"}]},{\"name\":\"blobURL\",\"dataType\":\"String\",\"maxLength\":512,\"validators\":[{\"maxLength\":512,\"name\":\"maxLength\"}]},{\"name\":\"dtExpiration\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"size\",\"dataType\":\"Int64\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"integer\"}]}]},{\"shortName\":\"StorefrontEmployeeApproverLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"StorefrontEmployeeApproverLinks\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"storefrontID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]}],\"navigationProperties\":[{\"name\":\"storefront\",\"entityTypeName\":\"Storefront:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontEmployeeApproverLink_Storefront\",\"foreignKeyNames\":[\"bID\",\"storefrontID\"]},{\"name\":\"employee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontEmployeeApproverLink_Employee\",\"foreignKeyNames\":[\"bID\",\"employeeID\"]}]},{\"shortName\":\"StorefrontGroupEmployeeApproverLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"StorefrontGroupEmployeeApproverLinks\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"storefrontGroupID\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]}],\"navigationProperties\":[{\"name\":\"storefrontGroup\",\"entityTypeName\":\"StorefrontGroup:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontGroupEmployeeApproverLink_StorefrontGroup\",\"foreignKeyNames\":[\"bID\",\"storefrontGroupID\"]},{\"name\":\"employee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontGroupEmployeeApproverLink_Employee\",\"foreignKeyNames\":[\"bID\",\"employeeID\"]}]},{\"shortName\":\"StorefrontGroup\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"StorefrontGroups\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"defaultImageURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"hasDefaultAdditionalImages\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"defaultThemeID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"storefrontLabelText\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"companyLabelText\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"primaryImageID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontGroup_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"storefronts\",\"entityTypeName\":\"Storefront:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Storefront_StorefrontGroup\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"defaultTheme\",\"entityTypeName\":\"Theme:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontGroup_DefaultTheme\",\"foreignKeyNames\":[\"defaultThemeID\"]},{\"name\":\"primaryImage\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontGroup_PrimaryImage\",\"foreignKeyNames\":[\"bID\",\"primaryImageID\"]},{\"name\":\"storefrontGroupContactApproverLinks\",\"entityTypeName\":\"StorefrontGroupContactApproverLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"StorefrontGroupContactApproverLink_StorefrontGroup\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"storefrontGroupEmployeeApproverLinks\",\"entityTypeName\":\"StorefrontGroupEmployeeApproverLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"StorefrontGroupEmployeeApproverLink_StorefrontGroup\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"AssetLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"AssetLinks\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"assetID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"sharedAssetID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"isTemporary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isSharedAsset\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"assetTypeID\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"mediaTypeID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"businessID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"storefrontID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"companyID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"contactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"orderID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"orderItemID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"itemDefID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"dtAdded\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"themeID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"inboxID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"alertID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"alertHistoryID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"notes\",\"dataType\":\"String\",\"maxLength\":8000,\"validators\":[{\"maxLength\":8000,\"name\":\"maxLength\"}]},{\"name\":\"metaData\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]}],\"navigationProperties\":[{\"name\":\"shared_Asset\",\"entityTypeName\":\"shared_Asset:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_shared_Asset\",\"foreignKeyNames\":[\"sharedAssetID\"]},{\"name\":\"asset\",\"entityTypeName\":\"Asset:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_Asset\",\"foreignKeyNames\":[\"bID\",\"assetID\"]},{\"name\":\"mediaType\",\"entityTypeName\":\"shared_AssetMediaType:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_MediaType\",\"foreignKeyNames\":[\"assetTypeID\",\"mediaTypeID\"]},{\"name\":\"assetType\",\"entityTypeName\":\"shared_AssetType:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_AssetType\",\"foreignKeyNames\":[\"assetTypeID\"]},{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_Business\",\"foreignKeyNames\":[\"businessID\"]},{\"name\":\"storefront\",\"entityTypeName\":\"Storefront:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_Storefront\",\"foreignKeyNames\":[\"bID\",\"storefrontID\"]},{\"name\":\"company\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_Company\",\"foreignKeyNames\":[\"bID\",\"companyID\"]},{\"name\":\"contact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_Contact\",\"foreignKeyNames\":[\"bID\",\"contactID\"]},{\"name\":\"employee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_Employee\",\"foreignKeyNames\":[\"bID\",\"employeeID\"]},{\"name\":\"order\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_Order\",\"foreignKeyNames\":[\"bID\",\"orderID\"]},{\"name\":\"orderItem\",\"entityTypeName\":\"OrderItem:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_OrderItem\",\"foreignKeyNames\":[\"bID\",\"orderItemID\"]},{\"name\":\"itemDef\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_ItemDef\",\"foreignKeyNames\":[\"bID\",\"itemDefID\"]},{\"name\":\"theme\",\"entityTypeName\":\"Theme:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_Theme\",\"foreignKeyNames\":[\"themeID\"]},{\"name\":\"inbox\",\"entityTypeName\":\"Inbox:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_Inbox\",\"foreignKeyNames\":[\"bID\",\"inboxID\"]},{\"name\":\"alert\",\"entityTypeName\":\"AlertQueue:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_Alert\",\"foreignKeyNames\":[\"bID\",\"alertID\"]},{\"name\":\"alertHistory\",\"entityTypeName\":\"AlertHistory:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetLink_AlertHistory\",\"foreignKeyNames\":[\"bID\",\"alertHistoryID\"]}]},{\"shortName\":\"shared_Asset\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"shared_Assets\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"assetURL\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":1024,\"validators\":[{\"name\":\"required\"},{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"assetTypeID\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"mediaTypeID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"refCount\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"dtUploaded\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"maxLength\":128,\"validators\":[{\"maxLength\":128,\"name\":\"maxLength\"}]},{\"name\":\"thumbnailURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"lowResImageURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"highResImageURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"height\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"width\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"aspectRatio\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"originalFileName\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"description\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"duration\",\"dataType\":\"Time\",\"validators\":[{\"name\":\"duration\"}]},{\"name\":\"dtOriginal\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"copyright\",\"dataType\":\"String\",\"maxLength\":64,\"validators\":[{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"metaData\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"size\",\"dataType\":\"Int64\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"integer\"}]}],\"navigationProperties\":[{\"name\":\"tags\",\"entityTypeName\":\"shared_AssetTag:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"shared_AssetTag_Asset\",\"invForeignKeyNames\":[\"iD\"]},{\"name\":\"assetCategoryLinks\",\"entityTypeName\":\"shared_AssetCategoryLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"shared_AssetCategoryLink_Asset\",\"invForeignKeyNames\":[\"iD\"]},{\"name\":\"assetType\",\"entityTypeName\":\"shared_AssetType:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"shared_Asset_AssetType\",\"foreignKeyNames\":[\"assetTypeID\"]},{\"name\":\"mediaType\",\"entityTypeName\":\"shared_AssetMediaType:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"shared_Asset_MediaType\",\"foreignKeyNames\":[\"assetTypeID\",\"mediaTypeID\"]}]},{\"shortName\":\"shared_AssetTag\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"shared_AssetTags\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Int64\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"integer\"}]},{\"name\":\"assetID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"tag\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"weight\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]}],\"navigationProperties\":[{\"name\":\"asset\",\"entityTypeName\":\"shared_Asset:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"shared_AssetTag_Asset\",\"foreignKeyNames\":[\"assetID\"]}]},{\"shortName\":\"shared_AssetParentChildLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"shared_AssetParentChildLinks\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"parentAssetID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"childAssetID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}]},{\"shortName\":\"shared_AssetCategoryLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_AssetCategoryLinks\",\"dataProperties\":[{\"name\":\"assetID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"categoryID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"asset\",\"entityTypeName\":\"shared_Asset:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"shared_AssetCategoryLink_Asset\",\"foreignKeyNames\":[\"assetID\"]},{\"name\":\"category\",\"entityTypeName\":\"shared_AssetCategory:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"shared_AssetCategoryLink_Category\",\"foreignKeyNames\":[\"categoryID\"]}]},{\"shortName\":\"shared_AssetCategory\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"shared_AssetCategories\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":512,\"validators\":[{\"name\":\"required\"},{\"maxLength\":512,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"assetCategoryLinks\",\"entityTypeName\":\"shared_AssetCategoryLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"shared_AssetCategoryLink_Category\",\"invForeignKeyNames\":[\"iD\"]}]},{\"shortName\":\"shared_AssetType\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_AssetTypes\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"assetType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":16,\"validators\":[{\"name\":\"required\"},{\"maxLength\":16,\"name\":\"maxLength\"}]},{\"name\":\"displayText\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":32,\"validators\":[{\"name\":\"required\"},{\"maxLength\":32,\"name\":\"maxLength\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"defaultMediaType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":128,\"validators\":[{\"name\":\"required\"},{\"maxLength\":128,\"name\":\"maxLength\"}]},{\"name\":\"thumbnailID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"assets\",\"entityTypeName\":\"Asset:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Asset_AssetType\",\"invForeignKeyNames\":[\"iD\"]},{\"name\":\"sharedAssets\",\"entityTypeName\":\"shared_Asset:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"shared_Asset_AssetType\",\"invForeignKeyNames\":[\"iD\"]},{\"name\":\"assetLinks\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"AssetLink_AssetType\",\"invForeignKeyNames\":[\"iD\"]},{\"name\":\"thumbnail\",\"entityTypeName\":\"shared_Asset:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"shared_AssetType_Thumbnail\",\"foreignKeyNames\":[\"thumbnailID\"]},{\"name\":\"mediaTypes\",\"entityTypeName\":\"shared_AssetMediaType:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"shared_AssetMediaType_AssetType\",\"invForeignKeyNames\":[\"iD\"]}]},{\"shortName\":\"Asset\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"Assets\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"assetURL\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":1024,\"validators\":[{\"name\":\"required\"},{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"assetTypeID\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"mediaTypeID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"refCount\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"dtUploaded\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"maxLength\":128,\"validators\":[{\"maxLength\":128,\"name\":\"maxLength\"}]},{\"name\":\"thumbnailURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"lowResImageURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"highResImageURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"height\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"width\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"aspectRatio\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"originalFileName\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"description\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"duration\",\"dataType\":\"Time\",\"validators\":[{\"name\":\"duration\"}]},{\"name\":\"dtOriginalUpload\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"copyright\",\"dataType\":\"String\",\"maxLength\":64,\"validators\":[{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"metaData\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"size\",\"dataType\":\"Int64\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"integer\"}]}],\"navigationProperties\":[{\"name\":\"assetType\",\"entityTypeName\":\"shared_AssetType:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Asset_AssetType\",\"foreignKeyNames\":[\"assetTypeID\"]},{\"name\":\"mediaType\",\"entityTypeName\":\"shared_AssetMediaType:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Asset_MediaType\",\"foreignKeyNames\":[\"assetTypeID\",\"mediaTypeID\"]},{\"name\":\"assetCategoryLinks\",\"entityTypeName\":\"AssetCategoryLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"AssetCategoryLink_Asset\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"tags\",\"entityTypeName\":\"AssetTag:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"AssetTag_Asset\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"parentLinks\",\"entityTypeName\":\"AssetParentChildLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"AssetParentChildLink_Child\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"childLinks\",\"entityTypeName\":\"AssetParentChildLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"AssetParentChildLink_Parent\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"assetLinks\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"AssetLink_Asset\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"shared_AssetMediaType\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"shared_AssetMediaTypes\",\"dataProperties\":[{\"name\":\"assetTypeID\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"iD\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"mediaType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":128,\"validators\":[{\"name\":\"required\"},{\"maxLength\":128,\"name\":\"maxLength\"}]},{\"name\":\"displayText\",\"dataType\":\"String\",\"maxLength\":64,\"validators\":[{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"isPartialMatchOK\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"defaultExtensions\",\"dataType\":\"String\",\"maxLength\":128,\"validators\":[{\"maxLength\":128,\"name\":\"maxLength\"}]},{\"name\":\"thumbnailID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"assetType\",\"entityTypeName\":\"shared_AssetType:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"shared_AssetMediaType_AssetType\",\"foreignKeyNames\":[\"assetTypeID\"]},{\"name\":\"thumbnail\",\"entityTypeName\":\"shared_Asset:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"shared_AssetMediaType_Thumbnail\",\"foreignKeyNames\":[\"thumbnailID\"]},{\"name\":\"assets\",\"entityTypeName\":\"Asset:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"Asset_MediaType\",\"invForeignKeyNames\":[\"assetTypeID\",\"iD\"]},{\"name\":\"sharedAssets\",\"entityTypeName\":\"shared_Asset:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"shared_Asset_MediaType\",\"invForeignKeyNames\":[\"assetTypeID\",\"iD\"]},{\"name\":\"assetLinks\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"AssetLink_MediaType\",\"invForeignKeyNames\":[\"assetTypeID\",\"iD\"]}]},{\"shortName\":\"AssetCategoryLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"AssetCategoryLinks\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"assetID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"categoryID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"asset\",\"entityTypeName\":\"Asset:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetCategoryLink_Asset\",\"foreignKeyNames\":[\"bID\",\"assetID\"]},{\"name\":\"category\",\"entityTypeName\":\"AssetCategory:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetCategoryLink_Category\",\"foreignKeyNames\":[\"bID\",\"categoryID\"]}]},{\"shortName\":\"AssetCategory\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"AssetCategories\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":128,\"validators\":[{\"name\":\"required\"},{\"maxLength\":128,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"assetCategoryLinks\",\"entityTypeName\":\"AssetCategoryLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"AssetCategoryLink_Category\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"AssetTag\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"AssetTags\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int64\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"integer\"}]},{\"name\":\"assetID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"tag\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"weight\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]}],\"navigationProperties\":[{\"name\":\"asset\",\"entityTypeName\":\"Asset:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetTag_Asset\",\"foreignKeyNames\":[\"bID\",\"assetID\"]}]},{\"shortName\":\"AssetParentChildLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"AssetParentChildLinks\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"parentAssetID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"childAssetID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"parent\",\"entityTypeName\":\"Asset:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetParentChildLink_Parent\",\"foreignKeyNames\":[\"bID\",\"parentAssetID\"]},{\"name\":\"child\",\"entityTypeName\":\"Asset:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AssetParentChildLink_Child\",\"foreignKeyNames\":[\"bID\",\"childAssetID\"]}]},{\"shortName\":\"OrderItem\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"OrderItems\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"orderID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"orderNumber\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"itemNumber\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"orderItemNumber\",\"dataType\":\"String\",\"maxLength\":20,\"validators\":[{\"maxLength\":20,\"name\":\"maxLength\"}]},{\"name\":\"itemID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"itemName\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"description\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"stationID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"quantity\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"inputUnitText\",\"dataType\":\"String\",\"maxLength\":32,\"validators\":[{\"maxLength\":32,\"name\":\"maxLength\"}]},{\"name\":\"priceSubTotal\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"priceDiscount\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"priceTotal\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"discountRate\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"discountOV\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"internalNotes\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"companyNotes\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]}],\"navigationProperties\":[{\"name\":\"itemDef\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderItem_ItemDef\",\"foreignKeyNames\":[\"bID\",\"itemID\"]},{\"name\":\"orderHeader\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderItem_OrderHeader\",\"foreignKeyNames\":[\"bID\",\"orderID\"]},{\"name\":\"orderItemMilestoneLinks\",\"entityTypeName\":\"OrderItemMilestoneLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderItemMilestoneLink_OrderItem\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"orderItemAttributes\",\"entityTypeName\":\"OrderItemAttribute:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderItemAttribute_OrderItem\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"itemInventoryLogs\",\"entityTypeName\":\"ItemInventoryLog:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemInventoryLog_OrderItem\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"ItemDef\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"ItemDefs\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"categoryID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"categoryPath\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"searchDescription\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"useSearchDescription\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"description\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"wODescription\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"imageURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"hasAdditionalImages\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"tags\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"sKUs\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"productType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.ProductType\"},{\"name\":\"itemGroupID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"isStockItem\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isShippingItem\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"activeStartDate\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"activeEndDate\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"hasImages\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isNegativeInvOK\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isInvDisplayed\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"taxClassificationID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"isExcludedFromDiscounts\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"entryFormID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"hasUnits\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"inputUnitID\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"pricingUnitID\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"showProductionTime\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"minProductionTime\",\"dataType\":\"Double\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"maxProductionTime\",\"dataType\":\"Double\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"deliveryType\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}],\"enumType\":\"Edm.Self.DeliveryType\"},{\"name\":\"otherCarrierName\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"lockArtworkAspectRatio\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"priceInfoText\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"priceInfoType\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}],\"enumType\":\"Edm.Self.ShowPriceType\"},{\"name\":\"primaryImageID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"primaryArtworkID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemDef_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"itemAttributes\",\"entityTypeName\":\"ItemAttribute:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemAttribute_ItemDef\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"taxClassification\",\"entityTypeName\":\"shared_TaxClassification:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemDef_TaxClassification\",\"foreignKeyNames\":[\"taxClassificationID\"]},{\"name\":\"category\",\"entityTypeName\":\"ItemCategory:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemDef_Category\",\"foreignKeyNames\":[\"bID\",\"categoryID\"]},{\"name\":\"entryForm\",\"entityTypeName\":\"ItemEntryForm:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemDef_EntryForm\",\"foreignKeyNames\":[\"bID\",\"entryFormID\"]},{\"name\":\"orderItems\",\"entityTypeName\":\"OrderItem:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderItem_ItemDef\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"storefrontItemLinks\",\"entityTypeName\":\"StorefrontItemLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"StorefrontItemLink_ItemDef\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"itemCategoryLinks\",\"entityTypeName\":\"ItemCategoryLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemCategoryLink_ItemDef\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"inputUnit\",\"entityTypeName\":\"shared_Unit:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemDef_InputUnit\",\"foreignKeyNames\":[\"inputUnitID\"]},{\"name\":\"pricingUnit\",\"entityTypeName\":\"shared_Unit:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemDef_PricingUnit\",\"foreignKeyNames\":[\"pricingUnitID\"]},{\"name\":\"itemInventory\",\"entityTypeName\":\"ItemInventory:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemDef_ItemInventory\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"itemInventoryLogs\",\"entityTypeName\":\"ItemInventoryLog:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemInventoryLog_ItemDef\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"itemGroup\",\"entityTypeName\":\"ItemChoiceGroup:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemChoiceGroup_Items\",\"foreignKeyNames\":[\"bID\",\"itemGroupID\"]},{\"name\":\"primaryImage\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemDef_PrimaryImage\",\"foreignKeyNames\":[\"bID\",\"primaryImageID\"]},{\"name\":\"primaryArtwork\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemDef_PrimaryArtwork\",\"foreignKeyNames\":[\"bID\",\"primaryArtworkID\"]}]},{\"shortName\":\"ItemAttribute\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"ItemAttributes\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"itemID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isSystem\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"sKU\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"inputType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.InputType\"},{\"name\":\"dataType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.DataType\"},{\"name\":\"isReadOnly\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isRequired\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"defaultValueFx\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"isList\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"listType\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}],\"enumType\":\"Edm.Self.ListType\"},{\"name\":\"listData\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"validationFx\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"isShownInGrid\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]}],\"navigationProperties\":[{\"name\":\"itemDef\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemAttribute_ItemDef\",\"foreignKeyNames\":[\"bID\",\"itemID\"]},{\"name\":\"orderItemAttributes\",\"entityTypeName\":\"OrderItemAttribute:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderItemAttribute_ItemAttribute\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"OrderItemAttribute\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"OrderItemAttributes\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"orderID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"orderItemID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"itemID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"itemAttributeID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"sKU\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"itemDataType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.DataType\"},{\"name\":\"isReadOnly\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isRequired\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"value\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"isList\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"listType\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}],\"enumType\":\"Edm.Self.ListType\"},{\"name\":\"listSelectionValue\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]}],\"navigationProperties\":[{\"name\":\"item\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderItemAttribute_Item\",\"foreignKeyNames\":[\"bID\",\"itemID\"]},{\"name\":\"itemAttribute\",\"entityTypeName\":\"ItemAttribute:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderItemAttribute_ItemAttribute\",\"foreignKeyNames\":[\"bID\",\"itemAttributeID\"]},{\"name\":\"orderItem\",\"entityTypeName\":\"OrderItem:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderItemAttribute_OrderItem\",\"foreignKeyNames\":[\"bID\",\"orderItemID\"]}]},{\"shortName\":\"shared_TaxClassification\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_TaxClassifications\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"tIC\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":15,\"validators\":[{\"name\":\"required\"},{\"maxLength\":15,\"name\":\"maxLength\"}]},{\"name\":\"description\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"defaultIsTaxable\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]}],\"navigationProperties\":[{\"name\":\"itemDefs\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemDef_TaxClassification\",\"invForeignKeyNames\":[\"iD\"]}]},{\"shortName\":\"ItemCategory\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"ItemCategories\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"parentCategoryID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"categoryPath\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"depth\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"isTopLevel\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"showOnNav\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]}],\"navigationProperties\":[{\"name\":\"subCategories\",\"entityTypeName\":\"ItemCategory:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemCategory_ParentCategory\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"parentCategory\",\"entityTypeName\":\"ItemCategory:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemCategory_ParentCategory\",\"foreignKeyNames\":[\"bID\",\"parentCategoryID\"]},{\"name\":\"itemDefs\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemDef_Category\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"itemCategoryLinks\",\"entityTypeName\":\"ItemCategoryLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemCategoryLink_ItemCategory\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"ItemCategoryLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"ItemCategoryLinks\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"categoryID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"itemID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"depth\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]}],\"navigationProperties\":[{\"name\":\"itemDef\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemCategoryLink_ItemDef\",\"foreignKeyNames\":[\"bID\",\"itemID\"]},{\"name\":\"itemCategory\",\"entityTypeName\":\"ItemCategory:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemCategoryLink_ItemCategory\",\"foreignKeyNames\":[\"bID\",\"categoryID\"]}]},{\"shortName\":\"ItemEntryForm\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"ItemEntryForms\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"description\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"template\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]}],\"navigationProperties\":[{\"name\":\"itemDefsThatUseMeAsEntryForm\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemDef_EntryForm\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"StorefrontItemLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"StorefrontItemLinks\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"storefrontID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"itemID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"storefront\",\"entityTypeName\":\"Storefront:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontItemLink_Storefront\",\"foreignKeyNames\":[\"bID\",\"storefrontID\"]},{\"name\":\"itemDef\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontItemLink_ItemDef\",\"foreignKeyNames\":[\"bID\",\"itemID\"]}]},{\"shortName\":\"shared_Unit\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_Units\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"dimType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":1,\"validators\":[{\"name\":\"required\"},{\"maxLength\":1,\"name\":\"maxLength\"}]},{\"name\":\"dimensionality\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"shortName\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":6,\"validators\":[{\"name\":\"required\"},{\"maxLength\":6,\"name\":\"maxLength\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":25,\"validators\":[{\"name\":\"required\"},{\"maxLength\":25,\"name\":\"maxLength\"}]},{\"name\":\"unitsPerBaseUnit\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"baseUnitsPerUnit\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"textMatch\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"itemDefsForInput\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemDef_InputUnit\",\"invForeignKeyNames\":[\"iD\"]},{\"name\":\"itemDefsForPricing\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemDef_PricingUnit\",\"invForeignKeyNames\":[\"iD\"]}]},{\"shortName\":\"ItemInventory\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"ItemInventories\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isEnabled\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"quantity\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]}]},{\"shortName\":\"ItemInventoryLog\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"ItemInventoryLogs\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"itemID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"dtAdjusted\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"startingQuantity\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"adjustment\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"endingQuantity\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"orderID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"orderItemID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"description\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"isOrderAdjustment\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isManualAdjustment\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]}],\"navigationProperties\":[{\"name\":\"employee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemInventoryLog_Employee\",\"foreignKeyNames\":[\"bID\",\"employeeID\"]},{\"name\":\"itemDef\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemInventoryLog_ItemDef\",\"foreignKeyNames\":[\"bID\",\"itemID\"]},{\"name\":\"orderHeader\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemInventoryLog_OrderHeader\",\"foreignKeyNames\":[\"bID\",\"orderID\"]},{\"name\":\"orderItem\",\"entityTypeName\":\"OrderItem:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"ItemInventoryLog_OrderItem\",\"foreignKeyNames\":[\"bID\",\"orderItemID\"]}]},{\"shortName\":\"ItemChoiceGroup\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"ItemChoiceGroups\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"zDTModified\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"groupType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.GroupType\"},{\"name\":\"isSingleChoice\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isMultipleChoice\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]}],\"navigationProperties\":[{\"name\":\"items\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"ItemChoiceGroup_Items\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"OrderItemMilestoneLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"OrderItemMilestoneLinks\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"orderID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"orderItemID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"milestoneID\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"isPending\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isCompleted\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isAssumedCompleted\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"dtDue\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtCompleted\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"completedByEmployeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"completedByContactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderItemMilestoneLink_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"order\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderItemMilestoneLink_Order\",\"foreignKeyNames\":[\"bID\",\"orderID\"]},{\"name\":\"orderItem\",\"entityTypeName\":\"OrderItem:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderItemMilestoneLink_OrderItem\",\"foreignKeyNames\":[\"bID\",\"orderItemID\"]},{\"name\":\"completedByEmployee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderItemMilestoneLink_CompletedByEmployee\",\"foreignKeyNames\":[\"bID\",\"completedByEmployeeID\"]},{\"name\":\"completedByContact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderItemMilestoneLink_CompletedByContact\",\"foreignKeyNames\":[\"bID\",\"completedByContactID\"]}]},{\"shortName\":\"Inbox\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"Inboxes\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"dtCreated\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"fromEmployeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"fromContactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"toEmployeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"toContactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"subject\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"body\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}]},{\"name\":\"dtRead\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtHandled\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"isRead\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isHandled\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isReadOrHandled\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Inbox_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"fromContact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Inbox_FromContact\",\"foreignKeyNames\":[\"bID\",\"fromContactID\"]},{\"name\":\"toContact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Inbox_ToContact\",\"foreignKeyNames\":[\"bID\",\"toContactID\"]},{\"name\":\"fromEmployee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Inbox_FromEmployee\",\"foreignKeyNames\":[\"bID\",\"fromEmployeeID\"]},{\"name\":\"toEmployee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Inbox_ToEmployee\",\"foreignKeyNames\":[\"bID\",\"toEmployeeID\"]},{\"name\":\"attachments\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"AssetLink_Inbox\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"AlertHistory\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"AlertHistories\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"dtCreated\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"deliveryAttempts\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"dtCompleted\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"wasDelivered\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"contactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"notificationMethod\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.NotificationMethodType\"},{\"name\":\"fromAddress\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"toAddress\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":256,\"validators\":[{\"name\":\"required\"},{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"separateMessages\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"subject\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"body\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}]},{\"name\":\"transmittedSize\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"deliveryNotesStr\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"deliveryNotes\",\"dataType\":\"Undefined\",\"validators\":[{\"name\":\"none\"}],\"rawTypeName\":\"Edm.List`1\"}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AlertHistory_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"contact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AlertHistory_Contact\",\"foreignKeyNames\":[\"bID\",\"contactID\"]},{\"name\":\"employee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"AlertHistory_Employee\",\"foreignKeyNames\":[\"bID\",\"employeeID\"]},{\"name\":\"attachments\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"AssetLink_AlertHistory\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"StorefrontGroupContactApproverLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"StorefrontGroupContactApproverLinks\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"storefrontGroupID\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"contactID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"storefrontGroup\",\"entityTypeName\":\"StorefrontGroup:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontGroupContactApproverLink_StorefrontGroup\",\"foreignKeyNames\":[\"bID\",\"storefrontGroupID\"]},{\"name\":\"contact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontGroupContactApproverLink_Contact\",\"foreignKeyNames\":[\"bID\",\"contactID\"]}]},{\"shortName\":\"OptionSet\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"Identity\",\"defaultResourceName\":\"OptionSets\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"bID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"storefrontID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"companyID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"contactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"optionLevel\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"optionValue\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OptionSet_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"company\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OptionSet_Company\",\"foreignKeyNames\":[\"bID\",\"companyID\"]},{\"name\":\"contact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OptionSet_Contact\",\"foreignKeyNames\":[\"bID\",\"contactID\"]},{\"name\":\"employee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OptionSet_Employee\",\"foreignKeyNames\":[\"bID\",\"employeeID\"]},{\"name\":\"optionDef\",\"entityTypeName\":\"shared_OptionDef:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OptionSet_OptionDef\",\"foreignKeyNames\":[\"name\"]},{\"name\":\"storefront\",\"entityTypeName\":\"Storefront:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OptionSet_Storefront\",\"foreignKeyNames\":[\"bID\",\"storefrontID\"]}]},{\"shortName\":\"shared_OptionDef\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_OptionDefs\",\"dataProperties\":[{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"isPartOfKey\":true,\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"dataType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.DataType\"},{\"name\":\"categoryPath\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":512,\"validators\":[{\"name\":\"required\"},{\"maxLength\":512,\"name\":\"maxLength\"}]},{\"name\":\"validOptionSet\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.ValidOptionSet\"},{\"name\":\"optionListValues\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"defaultOptionValue\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]},{\"name\":\"description\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]},{\"name\":\"isHidden\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isValidSystemOption\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isValidBusinessOption\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isValidStorefrontOption\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isValidEmployeeOption\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isValidCompanyOption\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isValidContactOption\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]}],\"navigationProperties\":[{\"name\":\"optionSets\",\"entityTypeName\":\"OptionSet:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OptionSet_OptionDef\",\"invForeignKeyNames\":[\"name\"]}]},{\"shortName\":\"OrderShipment\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"OrderShipments\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"orderID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"itemID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"itemName\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"priceSubTotal\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"priceDiscount\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"priceTotal\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"discountRate\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"discountOV\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"internalNotes\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]},{\"name\":\"customerNotes\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]},{\"name\":\"carrier\",\"dataType\":\"String\",\"maxLength\":128,\"validators\":[{\"maxLength\":128,\"name\":\"maxLength\"}]},{\"name\":\"costTotal\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"trackingNumber\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"dtShipped\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtArrived\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"carrierData\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]},{\"name\":\"isShipped\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isArrived\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"contactName\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"orderHeader\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderShipment_OrderHeader\",\"foreignKeyNames\":[\"bID\",\"orderID\"]},{\"name\":\"orderShipmentAddresses\",\"entityTypeName\":\"OrderShipmentAddress:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderShipmentAddress_OrderShipment\",\"invForeignKeyNames\":[\"bID\",\"iD\"]},{\"name\":\"orderShipmentLocators\",\"entityTypeName\":\"OrderShipmentLocator:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"OrderShipmentLocator_OrderShipment\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"OrderShipmentAddress\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"OrderShipmentAddresses\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"shipmentID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"formattedAddress\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isBilling\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isShipping\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isResidential\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isGeo\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isVerified\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"address\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"city\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"state\",\"dataType\":\"String\",\"maxLength\":32,\"validators\":[{\"maxLength\":32,\"name\":\"maxLength\"}]},{\"name\":\"postalCode\",\"dataType\":\"String\",\"maxLength\":16,\"validators\":[{\"maxLength\":16,\"name\":\"maxLength\"}]},{\"name\":\"country\",\"dataType\":\"String\",\"maxLength\":3,\"validators\":[{\"maxLength\":3,\"name\":\"maxLength\"}]},{\"name\":\"hasCoordinates\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"coordinates\",\"dataType\":\"Undefined\",\"validators\":[{\"name\":\"none\"}],\"rawTypeName\":\"Edm.Geography\"},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"orderShipment\",\"entityTypeName\":\"OrderShipment:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderShipmentAddress_OrderShipment\",\"foreignKeyNames\":[\"bID\",\"shipmentID\"]}]},{\"shortName\":\"OrderShipmentLocator\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"OrderShipmentLocators\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"shipmentID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"categoryType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.LocatorCategoryType\"},{\"name\":\"categorySubType\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PhoneType\"},{\"name\":\"formattedLocator\",\"dataType\":\"String\",\"maxLength\":512,\"validators\":[{\"maxLength\":512,\"name\":\"maxLength\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isPhone\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isEmail\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isURL\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isTwitter\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isFacebook\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isText\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"orderShipment\",\"entityTypeName\":\"OrderShipment:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderShipmentLocator_OrderShipment\",\"foreignKeyNames\":[\"bID\",\"shipmentID\"]}]},{\"shortName\":\"OrderMilestoneLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"OrderMilestoneLinks\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"orderID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"milestoneID\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"itemsUsing\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"pendingCount\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"completedCount\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"minDTDue\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"maxDTDue\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"minDTCompleted\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"maxDTCompleted\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"completedByEmployeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"completedByContactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"isPending\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderMilestoneLink_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"order\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderMilestoneLink_Order\",\"foreignKeyNames\":[\"bID\",\"orderID\"]},{\"name\":\"completedByEmployee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderMilestoneLink_CompletedByEmployee\",\"foreignKeyNames\":[\"bID\",\"completedByEmployeeID\"]},{\"name\":\"completedByContact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"OrderMilestoneLink_CompletedByContact\",\"foreignKeyNames\":[\"bID\",\"completedByContactID\"]}]},{\"shortName\":\"StorefrontContactApproverLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"StorefrontContactApproverLinks\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"storefrontID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"contactID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"storefront\",\"entityTypeName\":\"Storefront:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontContactApproverLink_Storefront\",\"foreignKeyNames\":[\"bID\",\"storefrontID\"]},{\"name\":\"contact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"StorefrontContactApproverLink_Contact\",\"foreignKeyNames\":[\"bID\",\"contactID\"]}]},{\"shortName\":\"CompanyAddress\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"CompanyAddresses\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"companyID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"formattedAddress\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isBilling\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isShipping\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isResidential\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isGeo\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isVerified\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"address\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"city\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"state\",\"dataType\":\"String\",\"maxLength\":32,\"validators\":[{\"maxLength\":32,\"name\":\"maxLength\"}]},{\"name\":\"postalCode\",\"dataType\":\"String\",\"maxLength\":16,\"validators\":[{\"maxLength\":16,\"name\":\"maxLength\"}]},{\"name\":\"country\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":3,\"validators\":[{\"name\":\"required\"},{\"maxLength\":3,\"name\":\"maxLength\"}]},{\"name\":\"hasCoordinates\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"coordinates\",\"dataType\":\"Undefined\",\"validators\":[{\"name\":\"none\"}],\"rawTypeName\":\"Edm.Geography\"},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"company\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"CompanyAddress_Company\",\"foreignKeyNames\":[\"bID\",\"companyID\"]}]},{\"shortName\":\"CompanyLocator\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"CompanyLocators\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"companyID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"categoryType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.LocatorCategoryType\"},{\"name\":\"categorySubType\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PhoneType\"},{\"name\":\"formattedLocator\",\"dataType\":\"String\",\"maxLength\":512,\"validators\":[{\"maxLength\":512,\"name\":\"maxLength\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isPhone\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isEmail\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isURL\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isTwitter\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isFacebook\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isText\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"company\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"CompanyLocator_Company\",\"foreignKeyNames\":[\"bID\",\"companyID\"]}]},{\"shortName\":\"CompanyTaxExemption\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"CompanyTaxExemptions\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"companyID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"state\",\"dataType\":\"String\",\"maxLength\":3,\"validators\":[{\"maxLength\":3,\"name\":\"maxLength\"}]},{\"name\":\"certificateNumber\",\"dataType\":\"String\",\"maxLength\":50,\"validators\":[{\"maxLength\":50,\"name\":\"maxLength\"}]},{\"name\":\"dtExpiration\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"isValid\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isForAllStates\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"notes\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"CompanyTaxExemption_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"company\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"CompanyTaxExemption_Company\",\"foreignKeyNames\":[\"bID\",\"companyID\"]}]},{\"shortName\":\"BusinessAddress\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"BusinessAddresses\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"formattedAddress\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isBilling\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isShipping\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isResidential\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isGeo\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isVerified\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"address\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"city\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"state\",\"dataType\":\"String\",\"maxLength\":32,\"validators\":[{\"maxLength\":32,\"name\":\"maxLength\"}]},{\"name\":\"postalCode\",\"dataType\":\"String\",\"maxLength\":16,\"validators\":[{\"maxLength\":16,\"name\":\"maxLength\"}]},{\"name\":\"country\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":3,\"validators\":[{\"name\":\"required\"},{\"maxLength\":3,\"name\":\"maxLength\"}]},{\"name\":\"hasCoordinates\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"BusinessAddress_Business\",\"foreignKeyNames\":[\"bID\"]}]},{\"shortName\":\"BusinessLocator\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"BusinessLocators\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"categoryType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.LocatorCategoryType\"},{\"name\":\"categorySubType\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PhoneType\"},{\"name\":\"formattedLocator\",\"dataType\":\"String\",\"maxLength\":512,\"validators\":[{\"maxLength\":512,\"name\":\"maxLength\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isPhone\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isEmail\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isURL\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isTwitter\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isFacebook\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isText\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"BusinessLocator_Business\",\"foreignKeyNames\":[\"bID\"]}]},{\"shortName\":\"URLRegistration\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"URLRegistrations\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Guid\",\"isNullable\":false,\"defaultValue\":\"00000000-0000-0000-0000-000000000000\",\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"guid\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"dtCreated\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"dtExpires\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtLastAccess\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"useCount\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"maxUses\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"redirectURL\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"securityRoleID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"URLRegistration_Business\",\"foreignKeyNames\":[\"bID\"]}]},{\"shortName\":\"Facility\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"Facilities\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"eTag\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"isActive\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":255,\"validators\":[{\"name\":\"required\"},{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"hasAdditionalImages\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isSalesFacility\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isProductionFacility\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"imageURL\",\"dataType\":\"String\",\"maxLength\":1024,\"validators\":[{\"maxLength\":1024,\"name\":\"maxLength\"}]},{\"name\":\"primaryImageID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Facility_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"primaryImage\",\"entityTypeName\":\"AssetLink:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Facility_PrimaryImage\",\"foreignKeyNames\":[\"bID\",\"primaryImageID\"]}]},{\"shortName\":\"FacilityAddress\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"FacilityAddresses\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"formattedAddress\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isBilling\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isShipping\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isResidential\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isGeo\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isVerified\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"address\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"city\",\"dataType\":\"String\",\"maxLength\":255,\"validators\":[{\"maxLength\":255,\"name\":\"maxLength\"}]},{\"name\":\"state\",\"dataType\":\"String\",\"maxLength\":32,\"validators\":[{\"maxLength\":32,\"name\":\"maxLength\"}]},{\"name\":\"postalCode\",\"dataType\":\"String\",\"maxLength\":16,\"validators\":[{\"maxLength\":16,\"name\":\"maxLength\"}]},{\"name\":\"country\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":3,\"validators\":[{\"name\":\"required\"},{\"maxLength\":3,\"name\":\"maxLength\"}]},{\"name\":\"hasCoordinates\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"coordinates\",\"dataType\":\"Undefined\",\"validators\":[{\"name\":\"none\"}],\"rawTypeName\":\"Edm.Geography\"},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}]},{\"shortName\":\"FacilityLocator\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"FacilityLocators\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"categoryType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.LocatorCategoryType\"},{\"name\":\"categorySubType\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PhoneType\"},{\"name\":\"formattedLocator\",\"dataType\":\"String\",\"maxLength\":512,\"validators\":[{\"maxLength\":512,\"name\":\"maxLength\"}]},{\"name\":\"isPrimary\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isPhone\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isEmail\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"isURL\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isTwitter\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isFacebook\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isText\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"data\",\"dataType\":\"String\",\"maxLength\":4096,\"validators\":[{\"maxLength\":4096,\"name\":\"maxLength\"}]}]},{\"shortName\":\"Payment\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"Payments\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"status\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"statusText\",\"dataType\":\"String\",\"maxLength\":12,\"validators\":[{\"maxLength\":12,\"name\":\"maxLength\"}]},{\"name\":\"amount\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"authorizedAmount\",\"dataType\":\"Decimal\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"number\"}]},{\"name\":\"capturedAmount\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"isPendingCapture\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"isCancelled\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"paymentType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PaymentType\"},{\"name\":\"paymentTypeText\",\"dataType\":\"String\",\"maxLength\":6,\"validators\":[{\"maxLength\":6,\"name\":\"maxLength\"}]},{\"name\":\"cardType\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PaymentCardType\"},{\"name\":\"cardTypeText\",\"dataType\":\"String\",\"maxLength\":10,\"validators\":[{\"maxLength\":10,\"name\":\"maxLength\"}]},{\"name\":\"processor\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PaymentProcessorType\"},{\"name\":\"displayNumber\",\"dataType\":\"String\",\"maxLength\":32,\"validators\":[{\"maxLength\":32,\"name\":\"maxLength\"}]},{\"name\":\"companyID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"orderID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"token\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"dtAuthorized\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"dtAuthorizedTill\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtCaptured\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtDeAuthorized\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtVoided\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtRefunded\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]}],\"navigationProperties\":[{\"name\":\"company\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Payment_Company\",\"foreignKeyNames\":[\"bID\",\"companyID\"]},{\"name\":\"orderHeader\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Payment_OrderHeader\",\"foreignKeyNames\":[\"bID\",\"orderID\"]},{\"name\":\"paymentAttempts\",\"entityTypeName\":\"PaymentAttempt:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"PaymentAttempt_Payment\",\"invForeignKeyNames\":[\"bID\",\"iD\"]}]},{\"shortName\":\"PaymentAttempt\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"PaymentAttempts\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"storefrontID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"status\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PaymentStatusType\"},{\"name\":\"statusText\",\"dataType\":\"String\",\"maxLength\":9,\"validators\":[{\"maxLength\":9,\"name\":\"maxLength\"}]},{\"name\":\"dtSubmitted\",\"dataType\":\"DateTime\",\"isNullable\":false,\"defaultValue\":\"1900-01-01T06:00:00.000Z\",\"validators\":[{\"name\":\"required\"},{\"name\":\"date\"}]},{\"name\":\"dtClosed\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"attemptType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PaymentAttemptType\"},{\"name\":\"attemptTypeText\",\"dataType\":\"String\",\"maxLength\":21,\"validators\":[{\"maxLength\":21,\"name\":\"maxLength\"}]},{\"name\":\"amount\",\"dataType\":\"Decimal\",\"validators\":[{\"name\":\"number\"}]},{\"name\":\"paymentType\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PaymentType\"},{\"name\":\"paymentTypeText\",\"dataType\":\"String\",\"maxLength\":11,\"validators\":[{\"maxLength\":11,\"name\":\"maxLength\"}]},{\"name\":\"cardType\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PaymentCardType\"},{\"name\":\"cardTypeText\",\"dataType\":\"String\",\"maxLength\":10,\"validators\":[{\"maxLength\":10,\"name\":\"maxLength\"}]},{\"name\":\"processor\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"validators\":[{\"name\":\"required\"},{\"name\":\"string\"}],\"enumType\":\"Edm.Self.PaymentProcessorType\"},{\"name\":\"paymentID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"companyID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"orderID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"contactID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"processorRecordID\",\"dataType\":\"String\",\"maxLength\":50,\"validators\":[{\"maxLength\":50,\"name\":\"maxLength\"}]},{\"name\":\"processorResponse\",\"dataType\":\"String\",\"maxLength\":2048,\"validators\":[{\"maxLength\":2048,\"name\":\"maxLength\"}]},{\"name\":\"processorErrorCode\",\"dataType\":\"String\",\"maxLength\":50,\"validators\":[{\"maxLength\":50,\"name\":\"maxLength\"}]},{\"name\":\"processorErrorDescription\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"token\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"cVVChecked\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"cVVResponse\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"cVVResponseCode\",\"dataType\":\"String\",\"maxLength\":50,\"validators\":[{\"maxLength\":50,\"name\":\"maxLength\"}]},{\"name\":\"aVSChecked\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"streetAddress\",\"dataType\":\"String\",\"maxLength\":200,\"validators\":[{\"maxLength\":200,\"name\":\"maxLength\"}]},{\"name\":\"postalCode\",\"dataType\":\"String\",\"maxLength\":50,\"validators\":[{\"maxLength\":50,\"name\":\"maxLength\"}]},{\"name\":\"aVSResponse\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"aVSResponseCode\",\"dataType\":\"String\",\"maxLength\":50,\"validators\":[{\"maxLength\":50,\"name\":\"maxLength\"}]}],\"navigationProperties\":[{\"name\":\"company\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"PaymentAttempt_Company\",\"foreignKeyNames\":[\"bID\",\"companyID\"]},{\"name\":\"contact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"PaymentAttempt_Contact\",\"foreignKeyNames\":[\"bID\",\"contactID\"]},{\"name\":\"employee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"PaymentAttempt_Employee\",\"foreignKeyNames\":[\"bID\",\"employeeID\"]},{\"name\":\"orderHeader\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"PaymentAttempt_OrderHeader\",\"foreignKeyNames\":[\"bID\",\"orderID\"]},{\"name\":\"payment\",\"entityTypeName\":\"Payment:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"PaymentAttempt_Payment\",\"foreignKeyNames\":[\"bID\",\"paymentID\"]}]},{\"shortName\":\"shared_Action\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_Actions\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"displayText\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":128,\"validators\":[{\"name\":\"required\"},{\"maxLength\":128,\"name\":\"maxLength\"}]},{\"name\":\"needsBusiness\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"needsEmployee\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"needsStorefrontGroup\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"needsStorefront\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"needsCompany\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"needsContact\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"needsItemDef\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"needsOrder\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"needsMilestone\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"needsStation\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"needsAsset\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"canSendToEmail\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"canSendToSMS\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"canSendToInbox\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"canSendToFTP\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"sortOrder\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]}],\"navigationProperties\":[{\"name\":\"eventActionLinks\",\"entityTypeName\":\"shared_EventActionLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"shared_EventActionLink_Action\",\"invForeignKeyNames\":[\"iD\"]}]},{\"shortName\":\"shared_EventActionLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_EventActionLinks\",\"dataProperties\":[{\"name\":\"actionID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"eventID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]}],\"navigationProperties\":[{\"name\":\"action\",\"entityTypeName\":\"shared_Action:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"shared_EventActionLink_Action\",\"foreignKeyNames\":[\"actionID\"]},{\"name\":\"event\",\"entityTypeName\":\"shared_Event:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"shared_EventActionLink_Event\",\"foreignKeyNames\":[\"eventID\"]}]},{\"shortName\":\"shared_Event\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_Events\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":64,\"validators\":[{\"name\":\"required\"},{\"maxLength\":64,\"name\":\"maxLength\"}]},{\"name\":\"displayText\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":128,\"validators\":[{\"name\":\"required\"},{\"maxLength\":128,\"name\":\"maxLength\"}]},{\"name\":\"canSubscribe\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"showBusiness\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"showEmployee\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"showStorefrontGroup\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"showStorefront\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"showCompany\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"showContact\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"showItemDef\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"showOrder\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"showMilestone\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"showStation\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"sortOrder\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]}],\"navigationProperties\":[{\"name\":\"eventActionLinks\",\"entityTypeName\":\"shared_EventActionLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"shared_EventActionLink_Event\",\"invForeignKeyNames\":[\"iD\"]},{\"name\":\"eventRoleLinks\",\"entityTypeName\":\"shared_EventRoleLink:#Eagle.Models\",\"isScalar\":false,\"associationName\":\"shared_EventRoleLink_Event\",\"invForeignKeyNames\":[\"iD\"]}]},{\"shortName\":\"shared_EventRoleLink\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_EventRoleLinks\",\"dataProperties\":[{\"name\":\"eventID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"roleID\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"event\",\"entityTypeName\":\"shared_Event:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"shared_EventRoleLink_Event\",\"foreignKeyNames\":[\"eventID\"]},{\"name\":\"role\",\"entityTypeName\":\"webpages_Roles:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"shared_EventRoleLink_Role\",\"foreignKeyNames\":[\"roleID\"]}]},{\"shortName\":\"webpages_Roles\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"webpages_Roles\",\"dataProperties\":[{\"name\":\"roleId\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"roleName\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":512,\"validators\":[{\"name\":\"required\"},{\"maxLength\":512,\"name\":\"maxLength\"}]}]},{\"shortName\":\"shared_OrderMilestone\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_OrderMilestones\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":32,\"validators\":[{\"name\":\"required\"},{\"maxLength\":32,\"name\":\"maxLength\"}]},{\"name\":\"setValue\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"optional\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]}]},{\"shortName\":\"shared_OrderStatus\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"shared_OrderStatuses\",\"dataProperties\":[{\"name\":\"iD\",\"dataType\":\"Byte\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"byte\"}]},{\"name\":\"name\",\"dataType\":\"String\",\"isNullable\":false,\"defaultValue\":\"\",\"maxLength\":16,\"validators\":[{\"name\":\"required\"},{\"maxLength\":16,\"name\":\"maxLength\"}]},{\"name\":\"sortIndex\",\"dataType\":\"Byte\",\"validators\":[{\"name\":\"byte\"}]}]},{\"shortName\":\"Subscription\",\"namespace\":\"Eagle.Models\",\"autoGeneratedKeyType\":\"None\",\"defaultResourceName\":\"Subscriptions\",\"dataProperties\":[{\"name\":\"bID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"iD\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"isPartOfKey\":true,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"eventID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"actionID\",\"dataType\":\"Int16\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int16\"}]},{\"name\":\"businessID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"employeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"storefrontGroupID\",\"dataType\":\"Byte\",\"validators\":[{\"name\":\"byte\"}]},{\"name\":\"storefrontID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"companyID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"contactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"orderID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"itemDefID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"statusID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"milestoneID\",\"dataType\":\"Byte\",\"validators\":[{\"name\":\"byte\"}]},{\"name\":\"stationID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"notifiedEmployeeID\",\"dataType\":\"Int16\",\"validators\":[{\"name\":\"int16\"}]},{\"name\":\"notifiedContactID\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]},{\"name\":\"emailFromAddress\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"emailToAddress\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"sMSFromAddress\",\"dataType\":\"String\",\"maxLength\":32,\"validators\":[{\"maxLength\":32,\"name\":\"maxLength\"}]},{\"name\":\"sMSToAddress\",\"dataType\":\"String\",\"maxLength\":32,\"validators\":[{\"maxLength\":32,\"name\":\"maxLength\"}]},{\"name\":\"notifyByEmail\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"notifyBySMS\",\"dataType\":\"Boolean\",\"validators\":[{\"name\":\"bool\"}]},{\"name\":\"notifyByInbox\",\"dataType\":\"Boolean\",\"isNullable\":false,\"defaultValue\":false,\"validators\":[{\"name\":\"required\"},{\"name\":\"bool\"}]},{\"name\":\"subject\",\"dataType\":\"String\",\"maxLength\":256,\"validators\":[{\"maxLength\":256,\"name\":\"maxLength\"}]},{\"name\":\"message\",\"dataType\":\"String\",\"validators\":[{\"name\":\"string\"}]},{\"name\":\"occurrences\",\"dataType\":\"Int32\",\"isNullable\":false,\"defaultValue\":0,\"validators\":[{\"name\":\"required\"},{\"name\":\"int32\"}]},{\"name\":\"dtLastTriggered\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"dtExpires\",\"dataType\":\"DateTime\",\"validators\":[{\"name\":\"date\"}]},{\"name\":\"occurrencesRemaining\",\"dataType\":\"Int32\",\"validators\":[{\"name\":\"int32\"}]}],\"navigationProperties\":[{\"name\":\"business\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_Business\",\"foreignKeyNames\":[\"bID\"]},{\"name\":\"subscribedBusiness\",\"entityTypeName\":\"Business:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_SubscribedBusiness\",\"foreignKeyNames\":[\"businessID\"]},{\"name\":\"company\",\"entityTypeName\":\"Company:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_Company\",\"foreignKeyNames\":[\"bID\",\"companyID\"]},{\"name\":\"contact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_Contact\",\"foreignKeyNames\":[\"bID\",\"contactID\"]},{\"name\":\"employee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_Employee\",\"foreignKeyNames\":[\"bID\",\"employeeID\"]},{\"name\":\"event\",\"entityTypeName\":\"shared_Event:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_Event\",\"foreignKeyNames\":[\"eventID\"]},{\"name\":\"action\",\"entityTypeName\":\"shared_Action:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_Action\",\"foreignKeyNames\":[\"actionID\"]},{\"name\":\"itemDef\",\"entityTypeName\":\"ItemDef:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_ItemDef\",\"foreignKeyNames\":[\"bID\",\"itemDefID\"]},{\"name\":\"notifiedContact\",\"entityTypeName\":\"Contact:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_NotifiedContact\",\"foreignKeyNames\":[\"bID\",\"notifiedContactID\"]},{\"name\":\"notifiedEmployee\",\"entityTypeName\":\"Employee:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_NotifiedEmployee\",\"foreignKeyNames\":[\"bID\",\"notifiedEmployeeID\"]},{\"name\":\"orderHeader\",\"entityTypeName\":\"OrderHeader:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_OrderHeader\",\"foreignKeyNames\":[\"bID\",\"orderID\"]},{\"name\":\"orderMilestone\",\"entityTypeName\":\"shared_OrderMilestone:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_OrderMilestone\",\"foreignKeyNames\":[\"milestoneID\"]},{\"name\":\"storefront\",\"entityTypeName\":\"Storefront:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_Storefront\",\"foreignKeyNames\":[\"bID\",\"storefrontID\"]},{\"name\":\"storefrontGroup\",\"entityTypeName\":\"StorefrontGroup:#Eagle.Models\",\"isScalar\":true,\"associationName\":\"Subscription_StorefrontGroup\",\"foreignKeyNames\":[\"bID\",\"storefrontGroupID\"]}]}],\"resourceEntityTypeMap\":{\"AlertQueues\":\"AlertQueue:#Eagle.Models\",\"Businesses\":\"Business:#Eagle.Models\",\"Themes\":\"Theme:#Eagle.Models\",\"Storefronts\":\"Storefront:#Eagle.Models\",\"Companies\":\"Company:#Eagle.Models\",\"shared_Industries\":\"shared_Industry:#Eagle.Models\",\"shared_Origins\":\"shared_Origin:#Eagle.Models\",\"Contacts\":\"Contact:#Eagle.Models\",\"CompanyActivities\":\"CompanyActivity:#Eagle.Models\",\"ContactAddresses\":\"ContactAddress:#Eagle.Models\",\"ContactLocators\":\"ContactLocator:#Eagle.Models\",\"OrderHeaders\":\"OrderHeader:#Eagle.Models\",\"Employees\":\"Employee:#Eagle.Models\",\"EmployeeAddresses\":\"EmployeeAddress:#Eagle.Models\",\"EmployeeLocators\":\"EmployeeLocator:#Eagle.Models\",\"Users\":\"User:#Eagle.Models\",\"UserAuths\":\"UserAuth:#Eagle.Models\",\"Carts\":\"Cart:#Eagle.Models\",\"StorefrontEmployeeApproverLinks\":\"StorefrontEmployeeApproverLink:#Eagle.Models\",\"StorefrontGroupEmployeeApproverLinks\":\"StorefrontGroupEmployeeApproverLink:#Eagle.Models\",\"StorefrontGroups\":\"StorefrontGroup:#Eagle.Models\",\"AssetLinks\":\"AssetLink:#Eagle.Models\",\"shared_Assets\":\"shared_Asset:#Eagle.Models\",\"shared_AssetTags\":\"shared_AssetTag:#Eagle.Models\",\"shared_AssetParentChildLinks\":\"shared_AssetParentChildLink:#Eagle.Models\",\"shared_AssetCategoryLinks\":\"shared_AssetCategoryLink:#Eagle.Models\",\"shared_AssetCategories\":\"shared_AssetCategory:#Eagle.Models\",\"shared_AssetTypes\":\"shared_AssetType:#Eagle.Models\",\"Assets\":\"Asset:#Eagle.Models\",\"shared_AssetMediaTypes\":\"shared_AssetMediaType:#Eagle.Models\",\"AssetCategoryLinks\":\"AssetCategoryLink:#Eagle.Models\",\"AssetCategories\":\"AssetCategory:#Eagle.Models\",\"AssetTags\":\"AssetTag:#Eagle.Models\",\"AssetParentChildLinks\":\"AssetParentChildLink:#Eagle.Models\",\"OrderItems\":\"OrderItem:#Eagle.Models\",\"ItemDefs\":\"ItemDef:#Eagle.Models\",\"ItemAttributes\":\"ItemAttribute:#Eagle.Models\",\"OrderItemAttributes\":\"OrderItemAttribute:#Eagle.Models\",\"shared_TaxClassifications\":\"shared_TaxClassification:#Eagle.Models\",\"ItemCategories\":\"ItemCategory:#Eagle.Models\",\"ItemCategoryLinks\":\"ItemCategoryLink:#Eagle.Models\",\"ItemEntryForms\":\"ItemEntryForm:#Eagle.Models\",\"StorefrontItemLinks\":\"StorefrontItemLink:#Eagle.Models\",\"shared_Units\":\"shared_Unit:#Eagle.Models\",\"ItemInventories\":\"ItemInventory:#Eagle.Models\",\"ItemInventoryLogs\":\"ItemInventoryLog:#Eagle.Models\",\"ItemChoiceGroups\":\"ItemChoiceGroup:#Eagle.Models\",\"OrderItemMilestoneLinks\":\"OrderItemMilestoneLink:#Eagle.Models\",\"Inboxes\":\"Inbox:#Eagle.Models\",\"AlertHistories\":\"AlertHistory:#Eagle.Models\",\"StorefrontGroupContactApproverLinks\":\"StorefrontGroupContactApproverLink:#Eagle.Models\",\"OptionSets\":\"OptionSet:#Eagle.Models\",\"shared_OptionDefs\":\"shared_OptionDef:#Eagle.Models\",\"OrderShipments\":\"OrderShipment:#Eagle.Models\",\"OrderShipmentAddresses\":\"OrderShipmentAddress:#Eagle.Models\",\"OrderShipmentLocators\":\"OrderShipmentLocator:#Eagle.Models\",\"OrderMilestoneLinks\":\"OrderMilestoneLink:#Eagle.Models\",\"StorefrontContactApproverLinks\":\"StorefrontContactApproverLink:#Eagle.Models\",\"CompanyAddresses\":\"CompanyAddress:#Eagle.Models\",\"CompanyLocators\":\"CompanyLocator:#Eagle.Models\",\"CompanyTaxExemptions\":\"CompanyTaxExemption:#Eagle.Models\",\"BusinessAddresses\":\"BusinessAddress:#Eagle.Models\",\"BusinessLocators\":\"BusinessLocator:#Eagle.Models\",\"URLRegistrations\":\"URLRegistration:#Eagle.Models\",\"Facilities\":\"Facility:#Eagle.Models\",\"FacilityAddresses\":\"FacilityAddress:#Eagle.Models\",\"FacilityLocators\":\"FacilityLocator:#Eagle.Models\",\"Payments\":\"Payment:#Eagle.Models\",\"PaymentAttempts\":\"PaymentAttempt:#Eagle.Models\",\"shared_Actions\":\"shared_Action:#Eagle.Models\",\"shared_EventActionLinks\":\"shared_EventActionLink:#Eagle.Models\",\"shared_Events\":\"shared_Event:#Eagle.Models\",\"shared_EventRoleLinks\":\"shared_EventRoleLink:#Eagle.Models\",\"webpages_Roles\":\"webpages_Roles:#Eagle.Models\",\"shared_OrderMilestones\":\"shared_OrderMilestone:#Eagle.Models\",\"shared_OrderStatuses\":\"shared_OrderStatus:#Eagle.Models\",\"Subscriptions\":\"Subscription:#Eagle.Models\"}}" };

        manager.importEntities(importData);

        var q2 = new breeze.EntityQuery("OrderHeaders")
            .using(breeze.FetchStrategy.FromLocalCache);

        stop();
        manager.executeQuery(q2).then(function (data2) {
            var order = data2.results[0];
            //uncomment line below and the relationship is resolved
            //manager._linkRelatedEntities(order);

            var orderShipments = order.getProperty("orderShipments");
            ok(orderShipments.length > 0, "relationship not resolved - should have 1 order shipment");
        }).fail(testFns.handleFail).fin(start);
    });

    test("Detaching the parent modifies the in-cache children - D2460", function () {
        if (testFns.DEBUG_MONGO) {
            ok(true, "NA for Mongo - expand not YET supported");
            return;
        }

        var em = newEm();
        var q = EntityQuery.from("Employees").where("employeeID", "==", 1)
            .expand("orders");
        stop();
        em.executeQuery(q).then(function (data) {
            var employee = data.results[0];
            var q2 = new EntityQuery("Orders");
            var orders = em.executeQueryLocally(q2);
            var order = orders[0];
            employee.entityAspect.setDetached();
            ok(!em.hasChanges());
        }).fail(testFns.handleFail).fin(start);
    });

    test("initialization", function () {
        var em = new EntityManager("foo");
        var so = new SaveOptions();
        em.setProperties({
            queryOptions: new QueryOptions(),
            saveOptions: new SaveOptions(),
            validationOptions: new ValidationOptions()
        });
        ok(em);
    });

    test("createEmptyCopy", function() {
        var em = newEm();
        var em2 = em.createEmptyCopy();
        var q = EntityQuery.from("Customers").take(1);
        stop();
        em2.executeQuery(q).then(function(data) {
            ok(data.results.length === 1);
        }).fail(testFns.handleFail).fin(start);
    });
    
    test("export/import deleted", function() {

        var em = newEm();
        var custType = em.metadataStore.getEntityType("Customer");
        var cust1 = custType.createEntity();
        cust1.setProperty("companyName", "Test_js_1");
        cust1.setProperty("city", "Oakland");
        cust1.setProperty("rowVersion", 13);
        cust1.setProperty("fax", "510 999-9999");
        em.addEntity(cust1);
        var cust2 = custType.createEntity();
        cust2.setProperty("companyName", "Test_js_2");
        cust2.setProperty("city", "Oakland");
        cust2.setProperty("rowVersion", 13);
        cust2.setProperty("fax", "510 999-9999");
        em.addEntity(cust2);

        stop();
        em.saveChanges().then(function(sr) {
            var custs = sr.entities;
            ok(custs.length == 2);
            custs[0].entityAspect.setDeleted();
            var newName = testFns.morphString(custs[1].getProperty("companyName"));
            custs[1].setProperty("companyName", newName);
            return em.saveChanges();
        }).then(function(sr) {
            ok(sr.entities.length == 2);
            var exported = em.exportEntities(null, false);
            var em2 = newEm();
            em2.importEntities(exported);
        }).fail(function (e) {
            var x = e;
        }).fin(start);


    });

    test("export/import with custom metadata", function () {
        var jsonMetadata = {
            "metadataVersion": "1.0.5",
            "dataServices": [
                {
                    "serviceName": "api/Foo/",
                    "hasServerMetadata": false,
                    "jsonResultsAdapter": "webApi_default",
                    "useJsonp": false
                }
            ],
            "structuralTypes": [
                {
                    "shortName": "address",
                    "namespace": "YourNamespace",
                    "isComplexType": true,
                    "dataProperties": [
                        { "name": "street", "dataType": "String" },
                        { "name": "city", "dataType": "String" },
                        { "name": "country", "dataType": "String" }
                    ]
                },
                {
                    "shortName": "person",
                    "namespace": "YourNamespace",
                    "dataProperties": [
                        { "name": "id", "dataType": "Int32", isPartOfKey: true },
                        { "name": "name", "dataType": "String" },
                        { "name": "hobbies", "dataType": "String" },
                        { "name": "address", "complexTypeName": "address:#YourNamespace" }
                    ]
                }
            ]
        };

        var manager = new breeze.EntityManager();
        manager.metadataStore.importMetadata(jsonMetadata)

        var person = manager.createEntity('person', { id: 1 });
        person.getProperty("address").setProperty("street", "Sample Street");

        // console.log("Complex property is a circular datatype, cannot convert to JSON - that's fine")
        // JSON.stringify(person.address); // fails with error

        // console.log("... except that manager.exportEntities() doesn't handle that case!");
        var exportedMs = manager.metadataStore.exportMetadata();
        var exportedEm = manager.exportEntities(); // also fails
        var manager2 = new breeze.EntityManager();
        manager2.importEntities(exportedEm);
        var ents = manager2.getEntities();
        ok(ents.length === 1);
        var samePerson = ents[0];
        ok(samePerson.getProperty("id") === 1, "id should be 1");
        ok(samePerson.entityAspect.getPropertyValue("address.street") === "Sample Street", "street names should be the same");
        
    });

    test("export/import complexTypes", function () {
        
        var em = newEm();
        var em2 = newEm();
        var q = EntityQuery.from("Suppliers")
            .where("companyName", "startsWith", "P");
        stop();
        em.executeQuery(q).then(function(data) {

            var suppliers = data.results;
            var suppliersCount = suppliers.length;
            ok(suppliersCount > 0, "should be some suppliers");
            var orderType = em.metadataStore.getEntityType("Order");
            // we want to have our reconsituted em to have different ids than our current em.
            em.keyGenerator.generateTempKeyValue(orderType);
            var empType = em.metadataStore.getEntityType("Employee");
            var custType = em.metadataStore.getEntityType("Customer");
        
            var order1 = em.addEntity(orderType.createEntity());
            ok(!order1.entityAspect.wasLoaded);
            var emp1 = em.addEntity(empType.createEntity());
            ok(!emp1.entityAspect.wasLoaded);
            emp1.setProperty("lastName", "bar");
            var cust1 = em.createEntity("Customer", { companyName: "foo" });
            //var cust1 = em.addEntity(custType.createEntity());
            //cust1.setProperty("companyName", "foo");
            ok(!cust1.entityAspect.wasLoaded);
            order1.setProperty("employee", emp1);
            order1.setProperty("customer", cust1);
            var exportedEm = em.exportEntities(null, false);

            em2.importEntities(exportedEm);
            var suppliers = em2.getEntities("Supplier");
            ok(suppliers.length === suppliersCount, "should be same number of suppliers");
            var addedOrders = em2.getChanges(orderType, EntityState.Added);
            ok(addedOrders.length === 1, "should be 1 added order");
            var addedCusts = em2.getChanges(custType, EntityState.Added);
            ok(addedCusts.length === 1, "should be 1 added customer");
            var order1x = addedOrders[0];
            var cust1x = order1x.getProperty("customer");
            ok(cust1x, "should have found a customer");
            ok(cust1x.getProperty("companyName") === "foo", "CompanyName should be 'foo'");
            var emp1x = order1x.getProperty("employee");
            ok(emp1x, "should have found an employee");
            ok(emp1x.getProperty("lastName") === "bar", "LastName should be 'bar'");
        }).fail(testFns.handleFail).fin(start);
    });
    
    test("mergeStrategy.overwrite", function () {
        var queryOptions = new QueryOptions({
            mergeStrategy: MergeStrategy.OverwriteChanges,
            fetchStrategy: FetchStrategy.FromServer
        });

        var em = new EntityManager({ serviceName: testFns.serviceName, metadataStore: testFns.metadataStore, queryOptions: queryOptions });
        var q = EntityQuery.from("Customers").take(2).using(em);
        var val = Date.now().toString();
        stop();
        q.execute().then(function (data) {
            var custs = data.results;
            custs[0].setProperty("companyName", val);
            custs[1].setProperty("city", val);
            return q.execute();
        }).then(function (data2) {
            var custs2 = data2.results;
            var companyName = custs2[0].getProperty("companyName");
            var city = custs2[1].getProperty("city");
            ok(companyName != val);
            ok(city != val);
        }).fin(start);
        
    });

    test("mergeStrategy.overwriteChanges and change events", function () {
        if (testFns.DEBUG_MONGO) {
            ok(true, "NA for Mongo - expand not yet supported");
            return;
        }

        var em = newEm();
        em.queryOptions = em.queryOptions.using(MergeStrategy.OverwriteChanges);
        var alfredsID = '785efa04-cbf2-4dd7-a7de-083ee17b6ad2';
        var query = EntityQuery.from("Customers")
            .where(testFns.customerKeyName, "==", alfredsID);
        var entityChangedArgs = [];
        var hasChangesChangedArgs = [];

        em.entityChanged.subscribe(function (args) {
            entityChangedArgs.push(args);
            lastArgs = args;
            lastAction = args.entityAction;
            lastEntity = args.entity;
        });
        em.hasChangesChanged.subscribe(function (args) {
            hasChangesChangedArgs.push(args);
        });
        var customer;
        stop();
        query.using(em).execute().then(function (data) {
            ok(!em.hasChanges(), "should not have any changes");
            customer = data.results[0];
            customer.setProperty("companyName", "Foo");
            ok(em.hasChanges(), "now should have changes");
            hasChangesChangedArgs.length = 0;
            entityChangedArgs.length = 0;
            return query.using(em).execute();
        }).then(function (data2) {
            ok(!em.hasChanges(), "now should no longer have changes");
            ok(em.getChanges().length === 0, "getChanges should return 0 records");
            ok(hasChangesChangedArgs.length == 1, "hasChangeschanged should have been fired");
            // entityStateChange and propertyChange
            ok(entityChangedArgs.length == 2, "entityChanged should have been fired twice");
        }).fail(testFns.handleFail).fin(start);

    });

    test("hasChanges basic", function() {
        var em = newEm();
        var orderType = em.metadataStore.getEntityType("Order");
        
        var saveNeeded = false;
        var count = 0;
        em.hasChangesChanged.subscribe(function (args) {
            count = count + 1;
            saveNeeded = args.saveNeeded;
        });
        ok(count === 0, "count should be 0");
        ok(!em.hasChanges(), "should be no changes");
        var order1 = orderType.createEntity();
        em.attachEntity(order1);
        ok(count === 0, "count should be 0");
        ok(!em.hasChanges(), "should be no changes");
        var order2 = orderType.createEntity();
        em.addEntity(order2);
        ok(count === 1, "count should be 1");
        ok(em.hasChanges(), "should be changes");
        em.rejectChanges();
        ok(!em.hasChanges(), "should not be changes");

    });

    test("em.acceptChanges", function () {
        var em = newEm();
        var orderType = em.metadataStore.getEntityType("Order");

        var saveNeeded = false;
        
        ok(!em.hasChanges(), "should be no changes");
        var order1 = orderType.createEntity();
        em.attachEntity(order1);
        ok(!em.hasChanges(), "should be no changes");
        var order2 = orderType.createEntity();
        em.addEntity(order2);
        ok(em.hasChanges(), "should be changes");
        em.acceptChanges();
        ok(!em.hasChanges(), "should not be changes");

    });
    
    test("hasChanges with rejectChanges", function () {
        var em = newEm();
        var orderType = em.metadataStore.getEntityType("Order");

        var saveNeeded = false;
        var count = 0;
        em.hasChangesChanged.subscribe(function (args) {
            count = count + 1;
            saveNeeded = args.saveNeeded;
        });
        ok(count === 0, "count should be 0");
        ok(!em.hasChanges(), "should be no changes");
        var order1 = orderType.createEntity();
        em.attachEntity(order1);
        order1.entityAspect.setModified();      
        ok(count === 1, "count should be 1");
        ok(em.hasChanges(), "should be no changes");
        var order2 = orderType.createEntity();
        em.addEntity(order2);
        ok(count === 1, "count should be 1");
        ok(em.hasChanges(), "should be changes");
        order1.entityAspect.rejectChanges();
        ok(count === 1, "count should be 1");
        ok(em.hasChanges(), "should be changes");
        order2.entityAspect.rejectChanges();
        ok(count === 2, "count should be 1");
        ok(!em.hasChanges(), "should not be any changes");
    });
    
    test("hasChanges with query mods", function () {
        var em = newEm();

        var hasChanges = false;
        var count = 0;
        em.hasChangesChanged.subscribe(function (args) {
            count = count + 1;
            hasChanges = args.hasChanges;
        });
        ok(count === 0, "count should be 0");
        ok(!em.hasChanges(), "should be no changes");
        stop();
        EntityQuery.from("Customers").take(3).using(em).execute().then(function(data) {
            var custs = data.results;
            custs[0].setProperty("companyName", "xxx");
            custs[1].entityAspect.setDeleted();
            custs[2].entityAspect.setModified();
            ok(count === 1, "count should be 1");
            ok(hasChanges, "should have changes");
            ok(em.hasChanges(), "should be changes");
            em.rejectChanges();
            ok(count === 2, "count should be 2");
            ok(!em.hasChanges(), "should be no changes");
            ok(!hasChanges, " should not have changes");
        }).fail(testFns.handleFail).fin(start);
        
    });
    
    test("initialization error on first query", function () {
        var em = new EntityManager("foo");
        stop();
        em.executeQuery("xxx").then(function(x) {
            ok(false, "shouldn't get here");
        }).fail(function(e) {
            ok(e.message.indexOf("foo") >= 0, "error message should mention 'foo'");
        }).fin(start);
        
    });
    
    test("store-gen keys are always set by key generator on add to manager if they have default values", function () {

        var em = newEm();
        var orderEntityType = em.metadataStore.getEntityType("Order");
        var o1 = orderEntityType.createEntity();
        var tempOrderId = o1.getProperty(testFns.orderKeyName);
        ok(tempOrderId == 0, "should be 0");

        em.addEntity(o1);
        tempOrderId = o1.getProperty(testFns.orderKeyName);
        ok(tempOrderId !== 0, "should not be 0");
        var isTempKey = em.keyGenerator.isTempKey(o1.entityAspect.getKey())
        ok(isTempKey, "should be a tempKey");
        
        stop();
        em.saveChanges().then(function (saveResult) {
            orderId = o1.getProperty(testFns.orderKeyName);
            ok(orderId !== tempOrderId);
            var keyMappings = saveResult.keyMappings;
            ok(keyMappings.length === 1);
            var mapping = keyMappings[0];
            ok(mapping.tempValue === tempOrderId);
            ok(mapping.realValue === orderId);
        }).fail(testFns.handleFail).fin(start);
    });

    test("store-gen keys are not re-set by key generator upon add to manager", function() {

        var dummyOrderID = testFns.wellKnownData.dummyOrderID;
        var em = newEm();
        var orderEntityType = em.metadataStore.getEntityType("Order");
        var o1 = orderEntityType.createEntity();
        o1.setProperty(testFns.orderKeyName, dummyOrderID); // waste of time to set id; it will be replaced.
        var orderId = o1.getProperty(testFns.orderKeyName);
        ok(orderId === dummyOrderID);
        //ok(o1.OrderID() !== 42,
        //    "o1's original key, 42, should have been replaced w/ new temp key.");

        em.addEntity(o1);
        orderId = o1.getProperty(testFns.orderKeyName);
        ok(orderId === dummyOrderID);
        stop();
        em.saveChanges().then(function(saveResult) {
            orderId = o1.getProperty(testFns.orderKeyName);
            ok(orderId !== 42);
            var keyMappings = saveResult.keyMappings;
            ok(keyMappings.length === 1);
            var mapping = keyMappings[0];
            ok(mapping.tempValue === dummyOrderID);
            ok(mapping.realValue === orderId);
        }).fail(testFns.handleFail).fin(start);
    });
    
    test("key generator reset", function () {
        var em = newEm();
        var dummyKeyGenerator = function () {
            this.dummy = true;
        };
        em.setProperties({ keyGeneratorCtor: dummyKeyGenerator });
        ok(em.keyGenerator.dummy === true);


    });
    
    test("import results notification", function () {
        if (testFns.DEBUG_MONGO) {
            ok(true, "NA for Mongo - expand");
            return;
        }
        var em = newEm();
        var em2 = newEm();
        var alfredsID = '785efa04-cbf2-4dd7-a7de-083ee17b6ad2';
        var query = EntityQuery.from("Customers")
            .where(testFns.customerKeyName, "==", alfredsID)
            .expand("orders")
            .using(em);
        stop();
        var exportedEm;
        var exportedCustomer;
        var arrayChangedCount = 0;
        var adds;
        query.execute().then(function (data) {
            var customer = data.results[0];
            exportedCustomer = em.exportEntities([customer], false);
            exportedEm = em.exportEntities(null, false);
            em2.importEntities(exportedCustomer);
            var sameCustomer = em2.findEntityByKey(customer.entityAspect.getKey());
            var orders = sameCustomer.getProperty("orders");
            ok(orders.length === 0, "orders should be empty to start");
            orders.arrayChanged.subscribe(function (args) {
                arrayChangedCount++;
                adds = args.added;
            });
            var r = em2.importEntities(exportedEm);
            ok(r.entities, "should have an 'entities' property");
            ok(r.tempKeyMapping, "should have a 'tempKeyMapping' property")
            ok(arrayChangedCount == 1, "should only see a single arrayChanged event fired");
            ok(adds && adds.length > 1, "should have been multiple entities shown as added");
        }).fail(testFns.handleFail).fin(start);
    });
    
    test("getChanges", function () {
        var em = newEm();
        var orderType = em.metadataStore.getEntityType("Order");
        var empType = em.metadataStore.getEntityType("Employee");
        var custType = em.metadataStore.getEntityType("Customer");
        for (var i = 0; i < 5; i++) {
            em.addEntity(orderType.createEntity()).entityAspect.setUnchanged();
            em.addEntity(empType.createEntity()).entityAspect.setUnchanged();
        }
        for (var i = 0; i < 5; i++) {
            em.addEntity(orderType.createEntity()).entityAspect.setModified();
            em.addEntity(empType.createEntity());
        }
        var c1 = em.getChanges();
        ok(c1.length === 10);
        var c2 = em.getChanges("Order");
        ok(c2.length === 5);
        var c3 = em.getChanges([orderType, custType]);
        ok(c3.length === 5);
        var c4 = em.getChanges([orderType, empType]);
        ok(c4.length === 10);
        var c5 = em.getEntities(["Order"], EntityState.Modified);
        ok(c5.length === 5);
        var c6 = em.getEntities([orderType], EntityState.Added);
        ok(c6.length === 0);
        var c7 = em.getEntities(null, EntityState.Added);
        ok(c7.length === 5);
        var c8 = em.getEntities(null, [EntityState.Added, EntityState.Modified]);
        ok(c8.length === 10);
    });
    
    test("entityChanged event", function() {
        var em = newEm();
        var orderType = em.metadataStore.getEntityType("Order");
        var empType = em.metadataStore.getEntityType("Employee");
        var custType = em.metadataStore.getEntityType("Customer");

        var lastArgs, lastAction, lastEntity;
        em.entityChanged.subscribe(function(args) {
            changedArgs.push(args);
            lastArgs = args;
            lastAction = args.entityAction;
            lastEntity = args.entity;
        });
        var order = orderType.createEntity();
        var changedArgs = [];
        em.addEntity(order);
        ok(lastAction === EntityAction.Attach, "lastAction should have been 'Attach'");
        ok(lastEntity === order, "last entity is wrong");

        var emp = empType.createEntity();
        changedArgs = [];
        em.attachEntity(emp);
        ok(lastAction === EntityAction.Attach, "lastAction should have been 'Attach'");
        ok(lastEntity === emp, "last entity is wrong");

        emp.setProperty("lastName", "Smith");
        ok(lastAction === EntityAction.PropertyChange, "lastAction should have been 'Attach'");
        ok(lastEntity === emp, "last entity is wrong");
        ok(lastArgs.args.propertyName === "lastName", "PropertyName is wrong");

        changedArgs = [];
        emp.entityAspect.rejectChanges();
        ok(changedArgs[0].entityAction === EntityAction.EntityStateChange, "should have seen a entityState change");
        ok(changedArgs[1].entityAction === EntityAction.RejectChanges, "lastAction should have been 'RejectChanges'");
        ok(lastEntity === emp, "last entity is wrong");
        
        
        emp.setProperty("lastName", "Jones");
        changedArgs = [];
        emp.entityAspect.acceptChanges();
        ok(changedArgs[0].entityAction === EntityAction.EntityStateChange, "should have seen a entityState change");
        ok(changedArgs[1].entityAction === EntityAction.AcceptChanges, "lastAction should have been 'AcceptChanges'");
        ok(lastEntity === emp, "last entity is wrong");

        em.clear();
        ok(lastAction === EntityAction.Clear, "lastAction should have been 'Clear'");
        ok(lastEntity === undefined, "last entity should be undefined");
    });
    
    

    test("entityChanged event - 2", function() {
        var em = newEm();
        var changedArgs = [];
        var lastArgs, lastAction, lastEntity;
        em.entityChanged.subscribe(function(args) {
            changedArgs.push(args);
            lastArgs = args;
            lastAction = args.entityAction;
            lastEntity = args.entity;
        });
        var q = new EntityQuery()
            .from("Employees")
            .orderBy("lastName")
            .take(2);
        stop();
        em.executeQuery(q).then(function (data) {
            ok(changedArgs.length == 2);
            changedArgs.forEach(function(arg) {
                ok(arg.entityAction === EntityAction.AttachOnQuery, "should have seen an AttachOnQuery"); 
            });
            var emps = data.results;
            ok(emps.length == 2, "results.length should be 2");
            emps[0].setProperty("lastName", "Smith");
            changedArgs = [];
            return em.executeQuery(q);
        }).then(function(data1) {
            ok(data1.results.length == 2, "results.length should be 2");
            // default MergeStrategy is preserveChanges so only unmodified entities should get merged.
            ok(changedArgs.length == 1, "No merge should have occured");
            q = q.using(MergeStrategy.OverwriteChanges);
            changedArgs = [];
            return em.executeQuery(q);
        }).then(function(data2) {
            ok(data2.results.length == 2, "results.length should be 2");
            ok(changedArgs.length == 3); // two mergeOnQuery and 1 entityStateChange
            var moqCount = 0;
            var esCount = 0;
            changedArgs.forEach(function (arg) {
                if (arg.entityAction === EntityAction.MergeOnQuery) {
                    moqCount++;
                }
                if (arg.entityAction == EntityAction.EntityStateChange) {
                    esCount++;
                }
            });
            ok(moqCount == 2 && esCount == 1, "change args should be correct");
            start();
        }).fail(testFns.handleFail);
    });
    
    test("entityChanged event suppressed", function () {
        var em = newEm();
        var orderType = em.metadataStore.getEntityType("Order");
        var empType = em.metadataStore.getEntityType("Employee");
        var custType = em.metadataStore.getEntityType("Customer");

        var changedArgs = [];
        Event.enable("entityChanged", em, false);
        em.entityChanged.subscribe(function (args) {
            changedArgs.push(args);
        });
        var order = orderType.createEntity();
        em.addEntity(order);
        var emp = empType.createEntity();
        em.attachEntity(emp);
        emp.setProperty("lastName", "Smith");
        emp.entityAspect.rejectChanges();
        emp.setProperty("lastName", "Jones");
        emp.entityAspect.acceptChanges();
        em.clear();
        ok(changedArgs.length === 0, "no change events should have fired");
    });

    test("entityChanged event suppressed by function", function () {
        var em = newEm();
        var orderType = em.metadataStore.getEntityType("Order");
        var empType = em.metadataStore.getEntityType("Employee");
        var custType = em.metadataStore.getEntityType("Customer");
        em.tag = "foo";
        var changedArgs = [];
        Event.enable("entityChanged", em, function (em) {
            return em.tag === "enabled";
        });
        em.entityChanged.subscribe(function (args) {
            changedArgs.push(args);
        });
        var order = orderType.createEntity();
        em.addEntity(order);
        var emp = empType.createEntity();
        em.attachEntity(emp);
        emp.setProperty("lastName", "Smith");
        emp.entityAspect.rejectChanges();
        emp.setProperty("lastName", "Jones");
        emp.entityAspect.acceptChanges();
        em.clear();
        ok(changedArgs.length === 0, "no change events should have fired");
    });


    test("wasLoaded", function () {
        var em = newEm();
        var orderType = em.metadataStore.getEntityType("Order");
        var empType = em.metadataStore.getEntityType("Employee");
        var custType = em.metadataStore.getEntityType("Customer");
        var order1 = em.attachEntity(orderType.createEntity());
        ok(!order1.entityAspect.wasLoaded);
        var emp1 = em.attachEntity(empType.createEntity());
        ok(!emp1.entityAspect.wasLoaded);
        var q = new EntityQuery().from("Employees").take(2);
        stop();
        em.executeQuery(q, function (data) {
            ok(data.results.length == 2, "results.length should be 2");
            data.results.forEach(function (r) {
                ok(r.entityAspect.wasLoaded === true);
            });
            start();
        }).fail(testFns.handleFail);
    });

    test("persist entityMetadata", function () {
        var em = newEm();
        var ets = em.metadataStore.getEntityTypes();
        var dataServices = em.metadataStore.dataServices;
        var exportedStore = em.metadataStore.exportMetadata();
        var newMs = new MetadataStore();
        newMs.importMetadata(exportedStore);
        var exportedStore2 = newMs.exportMetadata();
        ok(exportedStore.length === exportedStore2.length, "exported lengths should be the same");
        var newEts = newMs.getEntityTypes();
        ok(ets.length == newEts.length);
        for (var i = 0; i < ets.length; i++) {
            var dataServices2 = newMs.dataServices;
            ok(dataServices.length === dataServices2.length, "service names are different");
            var et = ets[i];
            var et2 = newMs.getEntityType(et.name);

            ok(et.name === et2.name, et.name + " :names are different");
            ok(et.dataProperties.length === et2.dataProperties.length, "data properties not the same length");
            core.arrayZip(et.dataProperties, et2.dataProperties, function (dp1, dp2) {
                ok(dp1.name === dp2.name, "dp names are not the same:" + dp1.name + " vs. " + dp2.name);
                ok(dp1.validators.length === dp2.validators.length, "validators length is not the same");
            });
            if (et instanceof EntityType) {
                ok(et.keyProperties.length === et2.keyProperties.length, "key properties not the same length");
                ok(et.navigationProperties.length === et2.navigationProperties.length, "navigation properties not the same length");
                ok(et.defaultResourceName === et2.defaultResourceName, "defaultResource names are different");
                ok(et.autoGeneratedKeyType === et2.autoGeneratedKeyType, "auto generated key types are different");
                ok(et.concurrencyProperties.length === et2.concurrencyProperties.length, "concurrency properties not the same length");
            }
            
            ok(et.unmappedProperties.length === et2.unmappedProperties.length, "unmapped properties not the same length");
            ok(et.validators.length === et2.validators.length);
        }

    });


    test("persist entityManager", function () {
        var em = newEm();
        var orderType = em.metadataStore.getEntityType("Order");
        // we want to have our reconsituted em to have different ids than our current em.
        em.keyGenerator.generateTempKeyValue(orderType);
        var empType = em.metadataStore.getEntityType("Employee");
        var custType = em.metadataStore.getEntityType("Customer");
        var order1 = em.addEntity(orderType.createEntity());
        ok(!order1.entityAspect.wasLoaded);
        var emp1 = em.addEntity(empType.createEntity());
        ok(!emp1.entityAspect.wasLoaded);
        emp1.setProperty("lastName", "bar");
        var cust1 = em.createEntity("Customer", { companyName: "foo" });
        //var cust1 = em.addEntity(custType.createEntity());
        //cust1.setProperty("companyName", "foo");
        ok(!cust1.entityAspect.wasLoaded);
        order1.setProperty("employee", emp1);
        order1.setProperty("customer", cust1);
        var q = new EntityQuery().from("Employees").take(2);
        stop();
        var em2;
        em.executeQuery(q, function(data) {
            ok(data.results.length == 2, "results.length should be 2");
            var exportedEm = em.exportEntities(null, false);
            em2 = newEm();
            var r = em2.importEntities(exportedEm);
            // 5 = 3 created + 2 queried
            ok(r.entities.length === 5, "should have imported 5 entities");
            var keys = Object.keys(r.tempKeyMapping);
            ok(keys.length === 3, "should have 3 keys in the tempKeyMapping");
            var r2 = em2.executeQueryLocally(q);
            ok(r2.length === 2, "should return 2 records");
            var addedOrders = em2.getChanges(orderType, EntityState.Added);
            ok(addedOrders.length === 1, "should be 1 added order");
            var addedCusts = em2.getChanges(custType, EntityState.Added);
            ok(addedCusts.length === 1, "should be 1 added customer");
            var order1x = addedOrders[0];
            var cust1x = order1x.getProperty("customer");
            ok(cust1x, "should have found a customer");
            ok(cust1x.getProperty("companyName") === "foo", "CompanyName should be 'foo'");
            var emp1x = order1x.getProperty("employee");
            ok(emp1x, "should have found an employee");
            ok(emp1x.getProperty("lastName") === "bar", "LastName should be 'bar'");
        }).fail(testFns.handleFail).fin(start);
    });
    
   test("persist entityManager partial", function () {
        var em = newEm();
        var orderType = em.metadataStore.getEntityType("Order");
        // we want to have our reconsituted em to have different ids than our current em.
        em.keyGenerator.generateTempKeyValue(orderType);
        var empType = em.metadataStore.getEntityType("Employee");
        var custType = em.metadataStore.getEntityType("Customer");
        var order1 = em.addEntity(orderType.createEntity());
        ok(!order1.entityAspect.wasLoaded);
        var emp1 = em.addEntity(empType.createEntity());
        ok(!emp1.entityAspect.wasLoaded);
        emp1.setProperty("lastName", "bar");
        var cust1 = em.addEntity(custType.createEntity());
        cust1.setProperty("companyName", "foo");
        ok(!cust1.entityAspect.wasLoaded);
        // order1.setProperty("Employee", emp1);
        order1.setProperty("customer", cust1);
        var q = new EntityQuery().from("Customers").take(2);
        stop();
        var em2;
       em.executeQuery(q, function(data) {
           ok(data.results.length == 2, "results.length should be 2");
           var cust2 = data.results[0];
           var exportedEm = em.exportEntities([order1, cust1, cust2]);
           em2 = newEm();
           var r = em2.importEntities(exportedEm);
           ok(r.entities.length === 3, "should have imported 3 entities");
           var r2 = em2.executeQueryLocally(q);
           ok(r2.length === 2, "should return 2 records");
           var addedOrders = em2.getChanges(orderType, EntityState.Added);
           ok(addedOrders.length === 1, "should be 1 added order");
           var addedCusts = em2.getChanges(custType, EntityState.Added);
           ok(addedCusts.length === 1, "should be 1 added customer");
           var order1x = addedOrders[0];
           var cust1x = order1x.getProperty("customer");
           ok(cust1x, "should have found a customer");
           ok(cust1x.getProperty("companyName") === "foo", "CompanyName should be 'foo'");
       }).fail(testFns.handleFail).fin(start);
   });
    
   test("importEntities  can safely merge and preserve or overwrite pending changes", 4, function () {
           // D#2207
           var em1 = newEm();
           var customerType = em1.metadataStore
               .getEntityType("Customer");

           var cust1 = customerType.createEntity();
           var cust1Id = core.getUuid();
           cust1.setProperty(testFns.customerKeyName, cust1Id);
           cust1.setProperty("companyName","Foo");
           em1.attachEntity(cust1);

           var exports = em1.exportEntities();

           // As if em2 queried for same customer
           var em2 = newEm();
           var cust1b = customerType.createEntity();
           cust1b.setProperty(testFns.customerKeyName, cust1Id);
           cust1b.setProperty("companyName","Foo");
           em2.attachEntity(cust1b);

           // then the user changed it but hasn't saved.
           var changedName = "Changed name";
           cust1b.setProperty("companyName", changedName);

           // Import from em1
           em2.importEntities(exports);

           ok(cust1b.entityAspect.entityState.isModified(),
               "cust1b should still be in Modified state after import");

           // Fails: D#2207
           equal(cust1b.getProperty("companyName"), changedName, 
               core.formatString("should retain pending cust name change, '%1'", changedName));

       
           em2.importEntities(exports, 
             { mergeStrategy: MergeStrategy.OverwriteChanges });
       
           ok(cust1b.entityAspect.entityState.isUnchanged(),
                   "cust1b should be in Unchanged state after import");

        
           notEqual(cust1b.getProperty("companyName"), changedName,
               "customer should not retain pending name change");
               
    });

    test("Export changes to local storage and re-import", 5, function () {

        var em = newEm();

         // add a new Cust to the cache
         var newCust = em.addEntity(createCust(em, "Export/import safely #1"));
         // add some more
         em.addEntity(createCust(em, "Export/import safely #2"));
         em.addEntity(createCust(em, "Export/import safely #3"));

         var changes = em.getChanges();
         var changesExport = em.exportEntities(changes, null);

         ok(window.localStorage, "this browser supports local storage");

         var stashName = "stash_newTodos";
         window.localStorage.setItem(stashName, changesExport);

         em.clear();
         ok(em.getEntities().length === 0,
             "em should be empty after clearing it");

         var changesImport = window.localStorage.getItem(stashName);
         em.importEntities(changesImport);

         var entitiesInCache = em.getEntities();
         var restoreCount = entitiesInCache.length;
         equal(restoreCount, 3, "restored 3 new Custs from file");

         var restoredCust = entitiesInCache[0];
         var restoredState = restoredCust.entityAspect.entityState;

         ok(restoredState.isAdded(),
              core.formatString("State of restored first Cust %1 is %2", restoredCust.getProperty("companyName"), restoredState));

         ok(newCust !== restoredCust,
             "Restored Cust is not the same object as the original Cust");
    });

 

    function createCust(em, companyName) {
        var custType = em.metadataStore.getEntityType("Customer");
        var cust = custType.createEntity();
        cust.setProperty("companyName", companyName);
        return cust;
    }

    // Uncomment when doing FULL testing
//    test("persist entityManager - large data", function () {
//        var em1 = newEm();
//        var q = new EntityQuery().from("CustomersAndOrders");
//        stop();
//        em1.executeQuery(q).then(function (data) {
//            var entities1 = em1.getEntities();
//            var exportedMs = em1.metadataStore.exportMetadata();
//            var exportedEm = em1.exportEntities();
//            ok(exportedEm.length > 200000, "exported size is too small");
//            var em2 = EntityManager.importEntities(exportedEm);
//            var entities2 = em2.getEntities();

//            ok(entities1.length == entities2.length, "lengths should be the same");
//            var exportedMs2 = em2.metadataStore.exportMetadata();
//            var exportedEm2 = em2.exportEntities();
//            ok(exportedMs.length === exportedMs2.length, "exported metadata sizes should be the same: " + exportedMs.length + " vs " + exportedMs2.length);
//            ok(exportedEm.length === exportedEm2.length, "exported entity manager sizes should be the same: " + exportedEm.length + " vs " + exportedEm2.length);
//            start();
//        }).fail(testFns.handleFail);
//    });

})(breezeTestFns);